/*
*  Rubrica
*  file: addressbook.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <gnome.h>
#include <gconf/gconf-client.h>

#include "rubrica.h"
#include "main.h"
#include "addressbook.h"
#include "callbacks.h"
#include "prefer.h"
#include "interface.h"
#include "dialoghi.h"
#include "file.h"
#include "file_util.h"



RubricaAddressBook *addressbook = NULL;
gboolean have_addressbook = FALSE;


/*    rubrica_addressbook_new, pu essere richiamata una
      sola volta. Inizializza la variabile globale
      addressbook
*/
void rubrica_addressbook_new(void) 
{
  if (!have_addressbook)
    addressbook = (RubricaAddressBook *) g_malloc(sizeof(RubricaAddressBook));
  else
    return;
    
  if (addressbook == NULL)
    {
      g_error(_("Out of memory! can't allocate memory for addressbook"));
    }

  have_addressbook = TRUE;
}



void rubrica_addressbook_init(void)
{  
  addressbook->notebook = gtk_notebook_new();
  addressbook->pages = 0;                      /* the first void page, if a new
						  page is added, it will be the 
						  number pages
					       */
  addressbook->current = 0;                    
  addressbook->book = NULL;                   /* il notebook viene mostrato in 
						 interface.c
					      */
}



RubricaAddressBook *rubrica_get_addressbook(void)
{
  return addressbook;
}


void
rubrica_addressbook_destroy(void)
{
  GList *alias;
  
  alias = addressbook->book;
  while(alias)
    {
      if (alias->data)
	{
	  Rubrica *ab = (Rubrica*) alias->data;

	  rubrica_svuota_memoria(ab->rub);
	  
	  gtk_object_unref(GTK_OBJECT(ab->child));
	  gtk_object_unref(GTK_OBJECT(ab->clist));
	  gtk_object_unref(GTK_OBJECT(ab->label));		   

	  ab->rub = NULL;	
	}

      alias = g_list_next(alias);
    }  
}



/*  add into addressbook's notebook a new page with the 
    content of file "filename"

    return the number of new page
*/
gint rubrica_addressbook_add_page(gchar *filename)
{
  Rubrica   *rubrica;       /* the new addressbook item */
  GtkWidget *child;         /* notebook child, this will contain cards clist */
  GtkWidget *label;         /* notebook tab label */
  GtkWidget *clist;         /* the cards clist */
  gint       page;          /* the # of the new notebook's page */
  gchar     *path; 
  gchar     *fname = NULL;

  if (filename)
    {
      path  = g_dirname(filename);
      fname = g_basename(filename);
    }
  else
    path = rubrica_preferences_get_home();

  /*    do new addressbook, init and insert it in addressbooks list. 
	It will be related to new notebook page  
  */
  rubrica = rubrica_rubrica_new();

  /*    make notebook page's child 
   */
  child = rubrica_addressbook_new_view(&clist);

  /*    tab label 
	NOTA:
	l'etichetta da inserire nel notebook, NON DEVE ESSERE MOSTRATA
	DON'T SHOW the label that will be inserted in notebook  
  */  
  if (!fname)
    fname = N_("no name");
  label = gtk_label_new (fname);  

  /*      get the number of this new page 
   */
  page = rubrica_addressbook_get_pages();

  /*      store infos about this addressbook/page 
   */
  rubrica->rub      = NULL;
  rubrica->modified = FALSE;
  rubrica->empty    = TRUE; 
  rubrica->new      = TRUE;
  rubrica->name     = (fname ? g_strdup(fname) : NULL);
  rubrica->path     = (path  ? g_strdup(path)  : NULL);
  rubrica->child    = child;
  rubrica->clist    = clist;
  rubrica->label    = label;
  rubrica->items    = 0;
  rubrica->riga     = -1;
  rubrica->colonna  = 0;
  rubrica->pagina   = page;

  /*      append new addressbook in addressbook's book 
   */
  rubrica_addressbook_append_rubrica(rubrica); 

  gtk_notebook_append_page(GTK_NOTEBOOK(addressbook->notebook), child, label);  

  rubrica_addressbook_make_this_rubrica_current(rubrica);

  /*      make this page the current one 
   */
  rubrica_addressbook_set_current_page(page); 

  /*      increase pages counter. Il valore di
	  pages, indica la posizione in cui 
	  sar inserita la prossima nuova pagina 
  */
  rubrica_addressbook_inc_pages();            
  
  /*      switch to page number "page" 
  */
  gtk_notebook_set_page(GTK_NOTEBOOK(addressbook->notebook), page);  

  /*      return the number of new page 
   */
  return (gint) page;  
}


/*  makes a new page for notebook
*/
GtkWidget *rubrica_addressbook_new_view(GtkWidget **clst)
{
  GtkWidget *scroll; /* scrolled window, this is the child of notebook page  */
  GtkWidget *cards;  /* cards clist, the core of page */ 
  GtkWidget *label;  

  /*     SCROLLED WINDOW
  */
  scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scroll, "scrollwin");
  gtk_widget_ref (scroll);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), 
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (scroll);

  /*      CARDS CLIST 
   */
  cards = gtk_clist_new (4);
  *clst = cards;
  gtk_widget_set_name (cards, "cards");
  gtk_widget_ref (cards);
  gtk_object_set_data_full (GTK_OBJECT (scroll), "cards", cards,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_clist_set_column_width (GTK_CLIST (cards), 0, 20);
  gtk_clist_set_column_width (GTK_CLIST (cards), 1, 110);
  gtk_clist_set_column_width (GTK_CLIST (cards), 2, 110);
  gtk_clist_set_column_width (GTK_CLIST (cards), 3, 110);

  gtk_container_add (GTK_CONTAINER (scroll), cards);

  /*    active clist button
   */
  gtk_clist_column_title_active(GTK_CLIST (cards), 1);
  gtk_clist_column_title_active(GTK_CLIST (cards), 2);
  gtk_clist_column_title_active(GTK_CLIST (cards), 3);
  gtk_clist_column_titles_show (GTK_CLIST (cards));
  gtk_widget_set_events (cards, GDK_BUTTON_PRESS | GDK_2BUTTON_PRESS |
			 GDK_3BUTTON_PRESS | GDK_BUTTON_RELEASE);
  gtk_widget_show (cards);

  /*    Clist button, labels pixmap column, no label
  */
  label = gtk_label_new ("");
  gtk_clist_set_column_widget (GTK_CLIST (cards), 0, label);
  gtk_widget_show (label);

  /*    card label
   */
  label = gtk_label_new (_(" Card "));
  gtk_clist_set_column_widget (GTK_CLIST (cards), 1, label);
  gtk_widget_show (label);

  /*    First Name (nome proprio) label
   */
  label = gtk_label_new (_(" First Name "));
  gtk_clist_set_column_widget (GTK_CLIST (cards), 2, label);
  gtk_widget_show (label);

  /*    Last Name (cognome) label
  */
  label = gtk_label_new (_(" Last Name "));
  gtk_clist_set_column_widget (GTK_CLIST (cards), 3, label);
  gtk_widget_show (label);

  /*    CALLBACK
  */
  gtk_signal_connect(GTK_OBJECT (cards), "select_row",
		     GTK_SIGNAL_FUNC (on_schede_clist_select_row),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(cards), "button_press_event",
		     GTK_SIGNAL_FUNC(on_schede_clist_button_click),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(cards), "click_column",
		     GTK_SIGNAL_FUNC(on_clicked_column_cb),
		     NULL);

  return (GtkWidget*) scroll;
}


/*  clean the addressbook view
 */ 
void rubrica_addressbook_init_view(void)
{
  GtkCList *dati;
  
  /* pulisco la clist dati  */
  dati = rubrica_view_get_dati(); 
  gtk_clist_clear(GTK_CLIST(dati));    
}



/*
 *
 *  copia il contenuto delle variabili rubrica, clist, ecc, relative al 
 *  addressbook selezionato nelle corrispondenti variabili globali
 *
 *  copy the content of addressbook number page to global variables.
 *
*/
void rubrica_addressbook_make_this_page_current(gint page)
{
  Rubrica *r;

  r = rubrica_addressbook_get_rubrica(page);
  rubrica_addressbook_make_this_rubrica_current(r);
}

void rubrica_addressbook_make_this_rubrica_current(Rubrica *rub)
{
  GtkWidget *window;
  gchar *book;  
  
  window = rubrica_get_app();  
  
  /* addressbook->book is a list of lists and contains all addressbooks 
     opened. Each notebook page is one addressbook. Data contained in 
     list item contains ptr to real addressbook, plus some infos about 
     the addressbook.
  */  
  if (!rub)
    return;
  
  /* save the current value of rubrica */
  rubrica_save_rubrica();

  /* set rub content as current rubrica value */
  rubrica_copy_rubrica(rub);
  
  /* imposto la chiave RUBRICA_KEY_LAST_BOOK di gconf con
     path e file name del file attualmente visibile

     set gconf key-value of RUBRICA_KEY_LAST_BOOK to 
     path and file name of current used file
  */  
  rubrica_view_update_path(rub->path);

  if (rub->name)
    book = g_strdup_printf("%s/%s", rub->path, rub->name);
  else
    book = g_strdup_printf("%s/%s", rub->path, _("no name"));    

  rubrica_preferences_set_last_book(book);
  g_free(book);

  if (rubrica_is_modified())
    {
      gchar *tmp;

      tmp = g_strdup_printf("Rubrica -- %s changed --", rub->name);

      gtk_window_set_title(GTK_WINDOW(window), _(tmp));
      
      g_free(tmp);
    }
  else
    gtk_window_set_title(GTK_WINDOW(window), "Rubrica");
  
  rubrica_view_set_indirizzi(rub->items);    
}


/* 
 *  insert a new rubrica in addressbook list
 */

void rubrica_addressbook_append_rubrica(Rubrica *r) 
{ 
  addressbook->book  = g_list_append(addressbook->book, r); 
} 

/* 
 *  ritorna la rubrica associata alla pagina "page" del notebook
 *  return the addressbook in position number page 
 */
Rubrica *rubrica_addressbook_get_rubrica(gint page)
{
  GtkWidget *child;
  Rubrica *rub;
  GList *alias;

  RUBRICA_IS_ADDRESSBOOK_PAGE(page);

  alias = addressbook->book;
  if (!alias)
    return NULL;

  /* se la pagina  la numero 0, allora ritorno la prima 
     pagina di book (il primo elemento della lista)

     if page is 0, then return the first item of book list
  */
  if (page == 0)
    return (Rubrica *) alias->data;

  child = gtk_notebook_get_nth_page(GTK_NOTEBOOK(addressbook->notebook), page);

  while (alias)
    {
      rub = (Rubrica *) alias->data;

      if (rub->child == child)
	return (Rubrica *) rub;
      else
	alias = g_list_next(alias);     
    }

  return NULL;
}



/* 
 * return the number of addressbooks 
 * ritorna il numero di rubriche contenute nell'addressbook
 */
inline gint 
rubrica_addressbook_get_pages(void)
{
  return addressbook->pages;
}

/* 
 * set addressbook number to n 
 * imposta il numero di rubriche ad n
 */
inline void 
rubrica_addressbook_set_page(gint n)
{
  addressbook->pages = n;
}

/* 
 * set current addressbook to the nth 
 * imposta il numero di rubricha corrente all'nesima
 */
inline gint
rubrica_addressbook_get_current_page(void)
{
  return gtk_notebook_get_current_page(GTK_NOTEBOOK(addressbook->notebook));
}

inline void 
rubrica_addressbook_set_current_page(gint n)
{
  addressbook->current = n;
}


/* 
 * add an addressbook 
 * incrementa le pagine del notebook (chiamata quando 
 * si carica un nuovo file di rubrica)
 */
inline void 
rubrica_addressbook_inc_pages(void)
{
  addressbook->pages++;
}


/* 
 * delete an addressbook 
 * decrementa le pagine del notebook (un file 
 * stato chiuso)
 */
inline void 
rubrica_addressbook_dec_pages(void)
{
  addressbook->pages--;
}

/* 
 * get the notebook ptr
 */
GtkWidget *rubrica_addressbook_get_notebook(void)  
{
  return (GtkWidget*) addressbook->notebook;
}



/* 
 *  metodi per operare sugli elementi contenuti 
 *  nella struttuta data (GList *book)
 *  di addressbook
 */


/*
 *  save the ptr to rubrica and rubrica's state (modified, empty, ...) of 
 *  addressbook contained in page number "pos"
 *
 */
void rubrica_addressbook_set_info(GList *rub, 
				  gboolean new, gboolean mod, gboolean emp, 
				  gchar *path, gchar *name,
				  GtkWidget *child, GtkWidget *clist, GtkWidget *label,
				  gint riga, gint colonna, 
				  gboolean rub_flag, gboolean flags_flag, 
				  gboolean name_flag, gboolean widget_flag, 
				  gboolean xy_flag, gint pos)
{
  Rubrica *data;

  if (pos < 0)
    return;
  
  data = rubrica_addressbook_get_rubrica(pos);
  if (!data)
    {
      return;
    }
  
  if (rub_flag)
    {
      data->rub = rub;      
      data->items = g_list_length(rub);
    }
  if (flags_flag)
    {
      data->modified = mod;
      data->empty = emp;
      data->new = new;
      data->items = g_list_length(rub);
   }
  if (name_flag)
    {
      if (data->path)
	g_free(data->path);
      data->path = g_strdup(path);

      if (data->name)
	g_free(data->name);
      data->name = g_strdup(name);
  
      /* show new filename in notebook tab */
      gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(addressbook->notebook), 
				      data->child, data->name);      
    }
  if (widget_flag)
    {      
      data->child = child;
      data->clist = clist;
      data->label = label;
    }
  if (xy_flag)
    {
      if (riga != -1)	
	data->riga    = riga;
      if (colonna != -1)
	data->colonna = colonna;
    }
  
  data->pagina = pos;           
}



/* 
 * return path and filename of addressbook number page
 * if the selected page don't contain addressbook then return NULL
 *
 */
void rubrica_addressbook_get_path_and_name(gchar **path, gchar **name, gint page)
{ 
  Rubrica *data;
  
  data = rubrica_addressbook_get_rubrica(page);
  
  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");
      
      *path = NULL;
      *name = NULL;
      return;
    }

  if (path)
    *path = g_strdup(data->path);
  if (name)
    *name = g_strdup(data->name);
}


gint rubrica_addressbook_get_items(gint page)
{
  Rubrica *data;
  
  data = rubrica_addressbook_get_rubrica(page);
  
  if (!data)
    return -1;
  
  return data->items;
}


gint rubrica_addressbook_get_row(gint page)
{
  Rubrica *data;
  
  data = rubrica_addressbook_get_rubrica(page);
  
  if (!data)
    return -1;
  
  return data->riga;
}


gint rubrica_addressbook_get_column(gint page)
{
  Rubrica *data;
  
  data = rubrica_addressbook_get_rubrica(page);
  
  if (!data)
    return -1;
  
  return data->colonna;
}


gint rubrica_addressbook_get_page(gint page)
{
  Rubrica *data;
  
  data = rubrica_addressbook_get_rubrica(page);
  
  if (!data)
    return -1;
  
  return data->pagina;
}



/*
 *  save widget info of addressbook contained in page number "pos" 
 */
void rubrica_addressbook_set_widget_info(GtkWidget *child, GtkWidget * clist, 
					 GtkWidget *label, gint page) 
{
  rubrica_addressbook_set_info(NULL, 
			       FALSE, FALSE, FALSE, 
			       NULL, NULL, 
			       child, clist,label, 
			       -1, -1,
			       FALSE, FALSE, FALSE, TRUE, FALSE,
			       page);
}


void rubrica_addressbook_set_label_text(gchar *str, gint page)
{
  Rubrica *data;

  data = rubrica_addressbook_get_rubrica(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }
  
  gtk_label_set_text(GTK_LABEL(data->label), str); 
}


void rubrica_addressbook_set_items(gint items, gint page)
{
  Rubrica *data;

  data = rubrica_addressbook_get_rubrica(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;      
    }
  
  data->items = items;
}


/*    save path and name in addressbook number page
 */
void rubrica_addressbook_set_row_and_column(gint riga, gint colonna, gint page) 
{
  rubrica_addressbook_set_info(NULL, 
			       FALSE, FALSE, FALSE,
			       NULL, NULL, 
			       NULL, NULL, NULL, 
			       riga, colonna, 
			       FALSE, FALSE, FALSE, FALSE, TRUE,
			       page);
}


void rubrica_addressbook_set_row(gint riga, gint page)
{
  Rubrica *data;

  data = rubrica_addressbook_get_rubrica(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;      
    }
  
  data->riga = riga;
}


void rubrica_addressbook_set_column(gint column, gint page)
{
  Rubrica *data;

  data = rubrica_addressbook_get_rubrica(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;      
    }
  
  data->colonna = column;
}


/*    save path and name in addressbook number page
*/
void rubrica_addressbook_set_path_and_name(gchar *path, gchar *name, gint page)
{
  rubrica_addressbook_set_info(NULL, 
			       FALSE, FALSE, FALSE, 
			       path, name, 
			       NULL, NULL, NULL, 
			       -1, -1, 
			       FALSE, FALSE, TRUE, FALSE, FALSE,
			       page);
}


void rubrica_addressbook_set_path(gchar *path, gint page)
{
  Rubrica *data;

  data = rubrica_addressbook_get_rubrica(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }
  
  if (data->path)
    g_free(data->path);
  data->path = g_strdup(path);   
}


void rubrica_addressbook_set_name(gchar *name, gint page)
{
  Rubrica *data;
  
  data = rubrica_addressbook_get_rubrica(page);
  
  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }
  
  if (data->name)
    g_free(data->name);
  data->path = g_strdup(name);   
}


/*
 * store the new ptr to rubrica if the old one was changed
 */
void rubrica_addressbook_set_rubrica(GList *rub, gint page) 
{
  rubrica_addressbook_set_info(rub, 
			       FALSE, FALSE, FALSE, 
			       NULL, NULL, 
			       NULL, NULL, NULL, 
			       -1, -1, 
			       TRUE, FALSE, FALSE, FALSE, FALSE,
			       page);
}


/*
 * set the flags of rubrica stored in "page" position
 */
void rubrica_addressbook_set_flags(gboolean new, gboolean empty, gboolean modified, 
				   gint page) 
{
  rubrica_addressbook_set_info(NULL, 
			       new, empty, modified, 
			       NULL, NULL, 
			       NULL, NULL, NULL, 
			       -1, -1,
			       FALSE, TRUE, FALSE, FALSE, FALSE,
			       page);
}



/*
 * set one flag of the three of rubrica
 */
void rubrica_addressbook_set_flag(gboolean val, gboolean is_new, gboolean is_empty, 
				  gboolean is_modified, gint page) 
{
  Rubrica *data;

  data = rubrica_addressbook_get_rubrica(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }
  
  if (is_new)
    {
      data->new = val;      
    }
  if (is_empty)
    {
      data->empty = val;
    }
  if (is_modified)
    {
      data->modified = val;
    }
}


/*
 *  set flags and rubrica ptr
 */
void rubrica_addressbook_set_rubrica_and_flags(GList *rub, gboolean new, 
					       gboolean empty, gboolean modif,
					       gint page)
{
  rubrica_addressbook_set_info(rub, 
			       new, empty, modif, 
			       NULL, NULL, 
			       NULL, NULL, NULL, 
			       -1, -1,
			       TRUE, TRUE, FALSE, FALSE, FALSE, 
			       page);
}


gboolean rubrica_addressbook_heve_modified()
{
  GList *alias;
  Rubrica *rub;
  gboolean bool = FALSE;

  alias = addressbook->book;

  while (alias)
    {
      rub = (Rubrica *) alias->data;

      bool |=  rub->modified;

      alias = g_list_next(alias);     
    }

  return bool;  
}


void rubrica_addressbook_close(void)
{    
  Rubrica *rub;
  gint scelta, current;
  gchar *mes;
  
  current = gtk_notebook_get_current_page(GTK_NOTEBOOK((addressbook->notebook)));

  rub = rubrica_addressbook_get_rubrica(current);
  
  if (!rub)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");
      
      return;
    }

  if (rub->modified)
    {
      mes = g_strdup(_("Warning. Address book has changed.\nChanges not saved, will be lose.\nContinue?")); 
      
      scelta = confirm_file_close(mes);
      g_free(mes);
      
      switch(scelta)
	{
	case 0:
	  /* delete addressbook, go down an delete */
	  break;
	  
	case 1:
	default:
	  return;
	  break;
	}  
    }

  /*      remove this node from addressbook's list 
  */
  addressbook->book = g_list_remove(addressbook->book, rub); 

  rubrica_svuota_memoria(rub->rub);         /* delete rubrica's contents */
  if (rub->name)                            /* delete path and filename  */
    g_free(rub->name);                             
  if (rub->path)
    g_free(rub->path);  
         
  gtk_notebook_remove_page(GTK_NOTEBOOK(addressbook->notebook), current);

  /*      clear view window 
  */
  rubrica_addressbook_init_view();

  rubrica_addressbook_dec_pages();          /* and update pages counter */
}

