/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file: callbacks.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "modulo.h"
#include "main.h"
#include "callbacks.h"
#include "prefer.h"
#include "prefer_int.h"
#include "search_int.h"
#include "sort.h"
#include "interface.h"
#include "dialoghi.h"
#include "support.h"
#include "error.h"
#include "file.h"
#include "addressbook.h"
#include "import.h"
#include "import_int.h"
#include "preview_int.h"

#include "rubrica.h"


extern GtkWidget *popup;  /* menu popup */

void
on_new_file1_activate(GtkMenuItem *menuitem, gpointer user_data)
{ 
  rubrica_addressbook_add_page(NULL);
}


void	
on_open1_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
  rubrica_file_carica_file();
}


void
on_save1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_file_salva();
  rubrica_set_modified(FALSE);
}


void
on_save_as1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_file_salva_come(NULL,NULL);
  rubrica_set_modified(FALSE);
}

void	
on_close1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_addressbook_close();
}


void 
on_import_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
  GtkWidget *import;
  
  import = create_import_dialog();
  gtk_widget_show(import);
}

void 
on_export_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_export_data();
}

void 
on_preview_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  GtkWidget *RubricaPreviewFs;
 
  RubricaPreviewFs = create_RubricaPreview ();
  gtk_widget_show (RubricaPreviewFs);  
}

void
on_exit1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  exit_rubrica();
}


void 
on_undo_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  /* not yet implemented */
  rubrica_non_implementato();
}

void
on_redo_activate (GtkMenuItem *menuitem,  gpointer user_data)
{ 
  /* not yet implemented */
  rubrica_non_implementato();
}


void
on_cut1_activate (GtkMenuItem *menuitem, gpointer user_data)
{   
  rubrica_item_cut();  
}


void
on_copy1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_item_copy();
}


void
on_paste1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_item_paste();
}


void
on_pop_cut_activate (GtkMenuItem *menuitem, GdkEventButton   *event,
		     gpointer user_data)
{ 
  rubrica_item_cut();  
}

void
on_pop_copy_activate (GtkMenuItem *menuitem, GdkEventButton   *event,
		      gpointer user_data)
{
  rubrica_item_copy();
}

void
on_pop_paste_activate (GtkMenuItem *menuitem, GdkEventButton   *event,
		       gpointer user_data)
{ 
  rubrica_item_paste();
}


void
on_pop_modifica_activate (GtkMenuItem *menuitem,  gpointer user_data)

{
  GtkWidget *app, *bot;
  
  app = rubrica_get_app();
  
  bot = gtk_object_get_data(GTK_OBJECT(app),"bModifica");
  gtk_signal_emit_by_name(GTK_OBJECT(bot), "clicked");
}


void
on_alfabetica1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_ordinamento_set_alfabetico();
  rubrica_riordina_lista();
}

/*
void 
on_dimensione1_activate (GtkMenuItem *menuitem, gpointer user_data) 
{ 
  rubrica_ordinamento_set_grandezza(); 
  rubrica_riordina_lista();     
} 

void 
on_data1_activate (GtkMenuItem *menuitem, gpointer user_data) 
{ 
  rubrica_ordinamento_set_data(); 
} 
*/

void on_creation_date_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_ordinamento_set_creation_date();
  rubrica_riordina_lista();
}

void on_modification_date_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_ordinamento_set_modification_date();
  rubrica_riordina_lista();
}


void
on_crescente1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_ordinamento_set_crescente();
  rubrica_riordina_lista();
}


void
on_decrescente1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  rubrica_ordinamento_set_decrescente();
  rubrica_riordina_lista();
}


void
on_preferences1_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
  GtkWidget *app      = NULL;
  GtkWidget *prefer   = NULL;
  GConfClient *client = NULL;
  GtkWidget *notebook = NULL;
  
  app    = rubrica_get_app();
  client = rubrica_get_client();

  prefer = rubrica_preferences_get_preferencesbox();
  
  if (!prefer) 
    {

      prefer = create_preferences_dialog(client);

      gtk_object_set_data (GTK_OBJECT (app), "preferences_dialog", prefer);
    }
  else
    {
      /* I have pointer to preferences dialog
	 set notebook to first page
      */

      notebook = gtk_object_get_data(GTK_OBJECT(prefer), "notebook");   
      if (GTK_IS_NOTEBOOK(notebook))	
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);      
    } 

  gtk_widget_show(prefer);    
}


void
on_about1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  info();
}


void
on_calabria1_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  calabria();
}


void
on_dedicato_a1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  dedica();
}

void
on_localization_activate  (GtkMenuItem *menuitem, gpointer user_data)
{
  create_help_me ();
}

void
on_bug_activate  (GtkMenuItem *menuitem, gpointer user_data)
{
  create_bug_dialog();
}


void
on_developing1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  create_tododialog(); 
}


void
on_unfinished_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  afaire();
}


void
on_home_page_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  create_homepage();
}


void
on_email_me_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  create_email_dialog();
}


void
on_bNuovo_clicked (GtkButton *button, gpointer user_data)
{ 
  rubrica_addressbook_add_page(NULL);
}


void
on_bApri_clicked (GtkButton *button, gpointer user_data)
{
  rubrica_file_carica_file();
}


void
on_bSalva_clicked (GtkButton *button, gpointer user_data)
{
  rubrica_file_salva();
  rubrica_set_modified(FALSE);
}


void
on_bClose_clicked  (GtkButton *button, gpointer user_data)
{
  rubrica_addressbook_close();
}


void
on_bAggiungi_clicked (GtkButton *button, gpointer data)
{
  GtkWidget *modulo;
  
  modulo = modulo_rubrica_new(NUOVO_MODULO);
  gtk_widget_show(modulo);
}


void
on_bElimina_clicked  (GtkButton *button, gpointer data)
{
  GtkWidget *clist;
  GList *rubrica;

  gint scelta, row;
  gboolean cancellato;
  gchar *buffer, *mes;

  clist = rubrica_get_clist();

  rubrica = rubrica_get_rubrica();

  if (rubrica_is_empty())
    {
      rubrica_error_rubrica_empty();
      
      return;
    }
  
  row = rubrica_get_row();
  if (row == -1)
    rubrica_error_selezionare_elemento();
  else 
    {
      gtk_clist_get_text(GTK_CLIST(clist), row, 1, &buffer);
      
      mes = g_strdup_printf(_("Do you want delete %s card?"), buffer);
      scelta = confirm_delete(mes);
      g_free(mes);
      
      switch (scelta)
	{
	case 0:
	  cancellato = rubrica_elimina_elemento(rubrica, row);
	  
	  if(!cancellato) 
	    /* c' stato un errore, e la scheda non  stata cancellata */
	    rubrica_error_cancellazione();
	  else 
	    {
	      /* la scheda  cancellata, bisogna:
		 -- aggiornare il numero di elementi 
		 (fatto da rubrica_elimina_elemento)
		 -- aggiornare l'elenco delle schede 
		 
		 elimino la scheda n info.riga 
		 e aggiorno la progressbar 	  */
	      rubrica_elimina_scheda(row);
	      rubrica_set_modified(TRUE);
	    }
	  break;
	case 1:   
	default:
	  break;
	}    
    }
}



/*  
    Modifica il contenuto di una scheda.
    Funzione richiamata da:
    pressione del bottone modifica sulla toolbar
    doppio click su un elemento della lista
    menu popup - voce modifica
*/
void
on_bModifica_clicked (GtkButton *button, gpointer user_data)
{
  RubricaItem *r;
  gint riga;

  if (rubrica_is_empty())
    {
      rubrica_error_rubrica_empty();
      
      return;
    }
  
  riga = rubrica_get_row();
  
  if (riga != -1)
    {
      r = rubrica_prendi_elemento(riga);
      
      modulo_rubrica_modifica_elemento(r);  
      rubrica_set_modified(TRUE);
    }
  else
    rubrica_error_selezionare_elemento();
}


void
on_bPrima_clicked (GtkButton *button, gpointer user_data)
{
  GtkVisibility visible;
  GtkWidget *clist;

  clist = rubrica_get_clist();

  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
  rubrica_set_row_selected(FALSE); 

  visible = gtk_clist_row_is_visible(GTK_CLIST(clist), 0);

  if ((visible == GTK_VISIBILITY_NONE) || 
      (visible == GTK_VISIBILITY_PARTIAL))
    gtk_clist_moveto(GTK_CLIST(clist), 0, 0, 0.8, 0); 
}


void
on_bUltima_clicked (GtkButton *button, gpointer user_data)
{
  GtkWidget *clist;
  GtkVisibility visible;
  gint len;

  clist = rubrica_get_clist();
  len = rubrica_get_items();

  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(clist), len-1, 0);
  rubrica_set_row_selected(FALSE); 

  visible = gtk_clist_row_is_visible(GTK_CLIST(clist), len-1);

  if ((visible == GTK_VISIBILITY_NONE) || 
      (visible == GTK_VISIBILITY_PARTIAL))
    gtk_clist_moveto(GTK_CLIST(clist), len-1, 0, 0.8, 0); 

}


void
on_bCerca_clicked (GtkButton *button, gpointer user_data)
{
  GtkWidget *app;
  GtkWidget *search;
  GConfClient *client; 

  app    = rubrica_get_app();
  client = rubrica_get_client();
  search = gtk_object_get_data (GTK_OBJECT (app), "search_dialog");

  if (!search) 
    search = create_search_dialog(client);
  else
    rubrica_search_dialog_clean();

  gtk_widget_show(search);
}


void
on_exit_clicked  (GtkButton *button, gpointer user_data)
{
  exit_rubrica();
}


void
on_bPrecedente_clicked (GtkButton *button, gpointer user_data)
{
  GtkWidget *clist;
  GtkVisibility visible;
  gint row;

  clist = rubrica_get_clist();
  row = rubrica_get_row();

  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(clist), row-1, 0);
  rubrica_set_row_selected(FALSE);

  visible = gtk_clist_row_is_visible(GTK_CLIST(clist), row-1);
  
  if ( (visible == GTK_VISIBILITY_NONE) || 
       (visible == GTK_VISIBILITY_PARTIAL))
    gtk_clist_moveto(GTK_CLIST(clist), row-1, 0, 0.2, 0); 

}


void
on_bSuccessivo_clicked (GtkButton *button, gpointer user_data)
{
  GtkWidget *clist;
  GtkVisibility visible;
  gint row;

  clist = rubrica_get_clist();
  row = rubrica_get_row();

  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(clist), row+1, 0);
  rubrica_set_row_selected(FALSE);

  visible = gtk_clist_row_is_visible(GTK_CLIST(clist), row+1);
  
  if ((visible == GTK_VISIBILITY_NONE) || 
      (visible == GTK_VISIBILITY_PARTIAL))
    gtk_clist_moveto(GTK_CLIST(clist), row+1, 0, 0.8, 0); 
}


void
on_schede_clist_select_row (GtkCList *clist, gint row, gint column,
			    GdkEventButton *event, gpointer user_data)
{
  GtkVisibility visible;

  g_return_if_fail(GTK_IS_CLIST(clist));

  if (row < 0) 
    return;

  if (rubrica_is_row_selected())
    {
      RubricaItem *r; 

      rubrica_set_row(row);
      rubrica_set_column(column);
      rubrica_set_range(1);

//      r = rubrica_prendi_elemento(row);
      r = (RubricaItem *) gtk_clist_get_row_data(GTK_CLIST(clist), row);
      if (r)
	rubrica_mostra_scheda(r);  

      visible = gtk_clist_row_is_visible(GTK_CLIST(clist), row);
  
      if ((visible == GTK_VISIBILITY_NONE) || 
	  (visible == GTK_VISIBILITY_PARTIAL))
	gtk_clist_moveto(GTK_CLIST(clist), row, 0, 0.5, 0); 

      return;      
    }

  if (event == NULL)
    return;
  
  rubrica_set_row(row);
  rubrica_set_column(column);
  rubrica_set_range(1);
   
  /* un solo click
     visualizzare l'elemento e dare all'utente la 
     possibilit di modificare o cancellare l'elemento selezionato
     
     one click
     rubrica must show item, the user can modify or delete the selected item
  */
  if (event->type == GDK_BUTTON_RELEASE)
    {
      RubricaItem *r; 
      
//      r = rubrica_prendi_elemento(row);
      r = (RubricaItem *) gtk_clist_get_row_data(GTK_CLIST(clist), row);
      if (r)
	rubrica_mostra_scheda(r);

      visible = gtk_clist_row_is_visible(GTK_CLIST(clist), row);
  
      if ((visible == GTK_VISIBILITY_NONE) || 
	  (visible == GTK_VISIBILITY_PARTIAL))
	gtk_clist_moveto(GTK_CLIST(clist), row, 0, 0.5, 0); 
      
      return;
    }
  
  /* doppio click
     l'utente deve modificare l'elemento selezionato
     
     double click 
     user will modify the selected item
  */
  if(event->type == GDK_2BUTTON_PRESS)
    {
      GtkWidget *bot, *app;
      
      app = rubrica_get_app();
      
      bot = gtk_object_get_data(GTK_OBJECT(app),"bModifica");  
      gtk_signal_emit_by_name(GTK_OBJECT(bot), "clicked");
      
      return;
    }
}


void
on_schede_clist_button_click (GtkCList *clist, GdkEventButton *event,
			      gpointer user_data)
{
  gint riga, colonna;
  
  g_return_if_fail(GTK_IS_CLIST(clist));
  g_return_if_fail(event != NULL);
  
  /* singolo click con il tasto destro del mouse
     visualizzo un menu
     
     one click with mouse's right button
     show a popup menu
  */ 
  if(event->button == 3)
    {
      gpointer data = NULL;
      gint range;
      /* recupero la riga selezionata 
	 get selected row */
      range = gtk_clist_get_selection_info(GTK_CLIST(clist), 
					   event->x, event->y, 
					   &riga, &colonna);
      
      /* se la selezione  avvenuta in una zona di clist contenente 
	 dati (riga o colonna), range = 1, se la selezione  avvenuta 
	 su una zona di clist che non contiene dati, range = 0.
	 
	 se range = 1, salvo la riga e la colonna attuali e seleziono 
	 la riga corrispondente
      */
      if (range)
	{
	  rubrica_set_row(riga);
	  rubrica_set_column(colonna);
	  gtk_clist_select_row(GTK_CLIST(clist), riga, colonna);
	  data = gtk_clist_get_row_data(GTK_CLIST(clist), riga);

	  rubrica_set_range(range);
	}
                
      /* richiamo il menu poppu 
	 call popup menu    */
      gnome_popup_menu_attach(popup,GTK_WIDGET(clist), data);
      
      return;
    }
}

/*
  TO DO
  ma sono utili?
  
  are they usefull, do i really need?
*/

void on_dati_clist_button_click(GtkCList    *clist,
				GdkEventButton   *event,
				gpointer user_data)
{
  
}



void
on_dati_clist_select_row (GtkCList    *clist,
			  gint row,
			  gint column,
			  GdkEventButton   *event,
			  gpointer user_data)
{
}

/*  --------------------------------------------------------------- */


/*   click su una colonna della clist
     
     +----+----+-----------+
     |    |    |  bla bla <=== click 
     +----+----+-----------+
     |    |    |           |
         ................
*/
void on_clicked_column_cb(GtkCList *clist, gint column, gpointer user_data)
{
  Ordinamento ord;
  GtkWidget  *notebook;
  Rubrica *rubrica;
  gint this_page;
   
  g_return_if_fail(GTK_IS_CLIST(clist));

  notebook = rubrica_addressbook_get_notebook();

  /* identifica la pagina del notebook */
  this_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));

  /* recupera la rubrica relativa alla pagina del notebook */
  rubrica = rubrica_addressbook_get_rubrica(this_page);

  ord = rubrica_get_ordinamento();

  /*    per colonna 1, 2, 3
   */
  switch (column)
    {
    case 1:
      rubrica_ordinamento_set_campo(SCHEDA);
      break;
    case 2:
      rubrica_ordinamento_set_campo(NOME);
      break;    
    case 3:  
      rubrica_ordinamento_set_campo(COGNOME);
      break;
    default:
      break;
    }
  
  if (ord.tipo == CRESCENTE)
    {
      rubrica_support_add_pixmap(clist, column, RUBRICA_UP);
      rubrica_ordinamento_set_decrescente();
    }
  else 
    {
      rubrica_support_add_pixmap(clist, column, RUBRICA_DOWN);
      rubrica_ordinamento_set_crescente();  
    }
  
  rubrica_riordina_lista(rubrica);
}


void on_switch_pages_cb (GtkNotebook *notebook, GtkNotebookPage *page,
			 gint page_num,  gpointer user_data)
{
  Rubrica *r;
  gint current_page;
  gint riga = 0;

  rubrica_set_range(0);

  current_page = rubrica_get_current_page();
  riga = rubrica_get_row();
  rubrica_addressbook_set_row(riga, current_page);

  rubrica_set_current_page(page_num);
  rubrica_addressbook_set_current_page(page_num);
  r = rubrica_addressbook_get_rubrica(page_num);
  rubrica_addressbook_make_this_rubrica_current(r); /* data in addressbook #page_num 
					               become globals */
  riga = rubrica_get_row();

  if (riga < 0)
    rubrica_addressbook_init_view();  /* clear data clist */
  else
    {
      GtkWidget *clist = rubrica_get_clist();

      rubrica_set_row_selected(TRUE);
      gtk_clist_select_row(GTK_CLIST(clist), riga, 0);
      rubrica_set_row_selected(FALSE); 
    }
}



/*  supporto alle callback
*/

void exit_rubrica(void)
{
  gchar *mes;
  gint scelta;
  gboolean exit;
  
  /*
    modificare la routine di uscita. Bisogna controllare che tutte le le rubrica 
    siano non modificate prima di uscire
  */
  exit = rubrica_preferences_get_exit();
  
  if (exit && !rubrica_is_modified() && !rubrica_other_are_modified())
    gtk_main_quit();
  else
    {
      if (rubrica_is_modified())
	mes = g_strdup(_("Warning. Address book has changed.\nChanges not saved will be loose.\nDo you want continue?"));
      else 
	if (rubrica_other_are_modified())
	  mes = g_strdup(_("Warning. Some Address book have changed.\nChanges not saved will be loose.\nDo you want continue?"));
      else
	mes = g_strdup(_("\nDo you want exit rubrica?"));
      
      scelta = confirm_exit(mes);
      g_free(mes);
      
      switch(scelta)
	{
	case 0:
	  gtk_main_quit();
	  break;
	  
	case 1:
	default:
	  break;
	}  
    }
}




