/*
*  Rubrica
*  file: dictionary.c
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <gconf/gconf-client.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "dictionary.h"
#include "prefer.h"
#include "main.h"

RubricaDictionary *rubrica_dict[52][2] = {
  {"Scheda",              "Card"             },
  {"cancellabile",        "deleting"         },
  {"editabile",           "modifying"        },
  {"creazione scheda",    "creation data"    },
  {"ultima modifica",     "last change"      },
  {"Dati",                "Data"             },
  {"Indirizzo",           "Address"          },
  {"Compagnia",           "Company"          },
  {"Lavoro",              "Work"             },
  {"Nome",                "First name"       }, 
  {"Secondo nome",        "Middle name"      },
  {"Cognome",             "Last name"        },
  {"Professione",         "Profession"       },
  {"Prefisso",            "Name prefix"      },
  {"Titolo",              "Title"            },
  {"Via",                 "Street"           },
  {"Numero",              "Street number"    },
  {"Cap",                 "Zip code"         },
  {"Citt",               "City"             },
  {"Provincia",           "Province"         },
  {"Stato",               "State"            },
  {"Nazione",             "Country"          },
  {"Indirizzi web",       "Web addresses"    },
  {"Web",                 "Web"              },
  {"Indirizzi email",     "Email addresses"  },
  {"EMail",               "EMail"            },
  {"Numeri telefonici",   "Telephone numbers"},
  {"Telefono",            "Telephone"        },
  {"tipo",                "type"             },
  {"Nome compagnia",      "Company name"     },
  {"Centralino",          "Operator"         },
  {"Fax",                 "Fax"              },
  {"Numero verde",        "Green"            },
  {"Assistenza clienti",  "Customer care"    },
  {"Ruolo",               "Assigment"        },
  {"Organizzazione",      "Organization"     },
  {"Dipartimento",        "Department"       },
  {"Sotto dipartimento",  "Sub department"   },
  {"Segretaria",          "Secretary"        },
  {"Telefono segretaria", "Secreatary phone" },
  {"Coniuge",             "Spouse"           },
  {"Coniugato",           "Married"          },
  {"Anniversario",        "Anniversary"      },
  {"Figli",               "Children"         },
  {"Note figli",          "Childen notes"    },
  {"Notes",               "Notes"            },
  {"Passatempi",          "Hobbies"          },
  {"Chiave publica",      "Public key"       },
  {"Data nascita",        "Birthday"         },
  {"Nome coniuge",        "Spouse name"      },
  {"Altre note",          "Other notes"      },
  {NULL, NULL}
};


void rubrica_dictionary_init(LANG language)
{ 
  rubrica_dictionary_get(language);
}


void rubrica_dictionary_get_italian(void)
{
  rubrica_dictionary_get(ITALIAN);
}

void rubrica_dictionary_get_english(void)
{
  rubrica_dictionary_get(ENGLISH);
}

void rubrica_dictionary_get_user_dictionary(void)
{
  gboolean err;

  err = rubrica_dictionary_get(USER);

  if (err)
    rubrica_dictionary_get(ITALIAN);
}


gboolean rubrica_dictionary_get(LANG language)
{
  GConfClient *client;
  gboolean err;

  client = rubrica_get_client();

  if (language == USER)
    {      
      err = rubrica_dictionary_load_user_dictionary(client);
      
      return err;
    }
  
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CARD, 
			  rubrica_dict[CARD][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_DELETE,
			  rubrica_dict[DELETING][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_MODIFY, 
			  rubrica_dict[MODIFYING][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CREATED, 
			  rubrica_dict[CREATED][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_LAST_CHANGE, 
			  rubrica_dict[LAST_CHANGE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_DATA, 
			  rubrica_dict[DATA][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_ADDRESS, 
			  rubrica_dict[ADDRESS][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_COMPANY, 
			  rubrica_dict[COMPANY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_WORK, 
			  rubrica_dict[WORK][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_FIRST_NAME, 
			  rubrica_dict[FIRST_NAME][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_MIDDLE_NAME, 
			  rubrica_dict[MIDDLE_NAME][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_LAST_NAME, 
			  rubrica_dict[LAST_NAME][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_PROFESSION, 
			  rubrica_dict[PROFESSION][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_NAME_PREFIX, 
			  rubrica_dict[NAME_PREFIX][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_TITLE, 
			  rubrica_dict[TITLE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_STREET, 
			  rubrica_dict[STREET][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_STREET_NUMBER, 
			  rubrica_dict[STREET_NUMBER][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_ZIP_CODE, 
			  rubrica_dict[ZIP_CODE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CITY, 
			  rubrica_dict[CITY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_PROVINCE, 
			  rubrica_dict[PROVINCE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_STATE, 
			  rubrica_dict[STATE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_COUNTRY, 
			  rubrica_dict[COUNTRY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_WEB_ADDRESS, 
			  rubrica_dict[WEB_ADDRESS][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_WEB, 
			  rubrica_dict[WEB][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_EMAIL_ADDRESS, 
			  rubrica_dict[EMAIL_ADDRESS][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_EMAIL, 
			  rubrica_dict[EMAIL][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_TELEPHONE_LIST, 
			  rubrica_dict[TELEPHONE_NUMBERS][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_TELEPHONE_NUMBER, 
			  rubrica_dict[TELEPHONE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_TELEPHONE_TYPE, 
			  rubrica_dict[TYPE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_COMPANY_NAME, 
			  rubrica_dict[COMPANY_NAME][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_OPERATOR, 
			  rubrica_dict[OPERATOR][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_FAX, 
			  rubrica_dict[FAX][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_GREEN, 
			  rubrica_dict[GREEN][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CUSTOMER_CARE, 
			  rubrica_dict[CUSTOMER_CARE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_ASSIGMENT, 
			  rubrica_dict[ASSIGMENT][language], NULL);  
  gconf_client_set_string(client, RUBRICA_KEY_DICT_ORGANIZATION, 
			  rubrica_dict[ORGANIZATION][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_DEPARTMENT, 
			  rubrica_dict[DEPARTMENT][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_SUB_DEPARTMENT, 
			  rubrica_dict[SUB_DEPARTMENT][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_SECRETARY, 
			  rubrica_dict[SECRETARY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_SECRETARY_PHONE, 
			  rubrica_dict[SECRETARY_PHONE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_SPOUSE_NAME, 
			  rubrica_dict[SPOUSE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_MARRIED, 
			  rubrica_dict[MARRIED][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_ANNIVERSARY, 
			  rubrica_dict[ANNIVERSARY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CHILDREN, 
			  rubrica_dict[CHILDREN][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CHILDREN_NOTES, 
			  rubrica_dict[CHILDREN_NOTES][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_HOBBY, 
			  rubrica_dict[HOBBIES][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_NOTES, 
			  rubrica_dict[NOTE][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_PUBLIC_KEY, 
			  rubrica_dict[PUBLIC_KEY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_BIRTHDAY, 
			  rubrica_dict[BIRTHDAY][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_CARD_NAME, 
			  rubrica_dict[CARD_NAME][language], NULL);
  gconf_client_set_string(client, RUBRICA_KEY_DICT_OTHER_NOTES, 
			  rubrica_dict[OTHER_NOTE][language], NULL);

  return FALSE;
}


void rubrica_dictionary_display_dictionary(GtkWidget *prefbox, GConfClient* client)
{
  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_card")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_CARD, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox), "entry_delete")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_DELETE, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_modify")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_MODIFY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_created")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_CREATED, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox), "entry_last_change")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_LAST_CHANGE, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_data")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_DATA, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_address")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_ADDRESS, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_company")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_COMPANY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_work")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_WORK, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox), "entry_first_name")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_FIRST_NAME, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_middle_name")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_MIDDLE_NAME, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_last_name")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_LAST_NAME, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_profession")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_PROFESSION, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_name_prefix")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_NAME_PREFIX, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_title")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_TITLE, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_street")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_STREET, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_street_number")), 
		     gconf_client_get_string(client,RUBRICA_KEY_DICT_STREET_NUMBER, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_zip_code")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_ZIP_CODE, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_city")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_CITY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_province")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_PROVINCE, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_state")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_STATE, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_country")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_COUNTRY, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_web_address")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_WEB_ADDRESS, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_web")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_WEB, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_email_address")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_EMAIL_ADDRESS,NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_email")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_EMAIL, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_tel_list")), 
		     gconf_client_get_string(client,RUBRICA_KEY_DICT_TELEPHONE_LIST,NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_telephone")), 
		     gconf_client_get_string(client,
					     RUBRICA_KEY_DICT_TELEPHONE_NUMBER, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_type")), 
		     gconf_client_get_string(client, 
					     RUBRICA_KEY_DICT_TELEPHONE_TYPE, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_company_name")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_COMPANY_NAME, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_operator")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_OPERATOR, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_fax")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_FAX, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_green")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_GREEN, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_customer_care")), 
		     gconf_client_get_string(client,RUBRICA_KEY_DICT_CUSTOMER_CARE, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_assigment")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_ASSIGMENT, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_organization")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_ORGANIZATION, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox), "entry_department")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_DEPARTMENT, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_sub_dep")), 
		     gconf_client_get_string(client,RUBRICA_KEY_DICT_SUB_DEPARTMENT,NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_secretary")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_SECRETARY, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_secretary_phone")), 
		     gconf_client_get_string(client,
					     RUBRICA_KEY_DICT_SECRETARY_PHONE, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_spouse_name")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_SPOUSE_NAME, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_married")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_MARRIED, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_anniversary")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_ANNIVERSARY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_children")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_CHILDREN, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_child_notes")), 
		     gconf_client_get_string(client,
					     RUBRICA_KEY_DICT_CHILDREN_NOTES, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_hobby")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_HOBBY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_notes")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_NOTES, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_public_key")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_PUBLIC_KEY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_birthday")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_BIRTHDAY, NULL));

  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_data(GTK_OBJECT(prefbox),"entry_name")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_CARD_NAME, NULL));

  gtk_entry_set_text(GTK_ENTRY
		     (gtk_object_get_data(GTK_OBJECT(prefbox),"entry_other_notes")), 
		     gconf_client_get_string(client, RUBRICA_KEY_DICT_OTHER_NOTES, NULL));
}


/*
  return TRUE if an error has occurred
  return FALSE if error hasn't occurred
 */
gboolean rubrica_dictionary_load_user_dictionary(GConfClient* client)
{
  xmlDocPtr doc;
  xmlNodePtr node;
  gchar *dictionary;
  gchar *language;
  gchar *home, *lang;
  gchar *dict_dir;
  
//  home       = g_get_home_dir();
  home       = rubrica_preferences_get_home();
  language   = rubrica_preferences_get_user_dictionary();  
  dict_dir   = g_strdup_printf("%s/.rubrica/dictionary", home);  
  dictionary = g_strdup_printf("%s/.rubrica/dictionary/%s", home, language); 
 
  if (!g_file_exists(dict_dir))
    mkdir(dict_dir, S_IRUSR | S_IWUSR | S_IXUSR);
      
  if (!g_file_exists(dictionary))
    {
      gchar *str;

      str = g_strdup_printf(_("The requested dictionary, %s, don't exist"), dictionary); 

//      rubrica_error_generic(str);
      g_warning(str);

      g_free(dictionary);
      g_free(str);

      return TRUE;
    }
  
  doc = xmlParseFile(dictionary);
  if (!doc)
    {
      g_warning(_("I can't read the %s dictionary"), dictionary);      
      return TRUE;
    }

   if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica_dictionary") != 0)
    {
      g_warning(_("This dictionary is corrupted\nHeader don't match with \"Rubrica_dictionary\"\nI can't read this dictionary"));
      xmlFreeDoc(doc);
      return TRUE;
    } 

   lang = xmlNodeGetLang(doc->root); 
   if (g_strcasecmp(lang, language) != 0)
     {
       g_warning(_("Language stored in file and requested language don't match"));
     }

   node = doc->root->childs;

   while (node)
     {
       rubrica_dictionary_parse_node(client, node);
       node = node->next;
     }   

   xmlFreeDoc(doc);

   return FALSE;
}

void rubrica_dictionary_parse_node(GConfClient *client, xmlNodePtr node)
{
  gchar *buffer;

  if (strcmp(node->name, "General") == 0)
    {
      buffer = xmlGetProp(node, "Card");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CARD, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Deleting");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_DELETE, 
			      buffer ? buffer : "", NULL);

      buffer = xmlGetProp(node, "Modifying");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_MODIFY, 
			      buffer ? buffer : "", NULL);

      buffer = xmlGetProp(node, "Created");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CREATED, 
			      buffer ? buffer : "", NULL);

      buffer = xmlGetProp(node, "LastChange");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_LAST_CHANGE, 
			      buffer ? buffer : "", NULL);

      buffer = xmlGetProp(node, "Data");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_DATA, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Address");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_ADDRESS, 
			      buffer ? buffer : "", NULL);     

      buffer = xmlGetProp(node, "Company");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_COMPANY, 
			      buffer ? buffer : "", NULL);

      buffer = xmlGetProp(node, "Work");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_WORK, 
			      buffer ? buffer : "", NULL);
    }
  else if (strcmp(node->name, "Personal") == 0)
    {
      buffer = xmlGetProp(node, "FirstName");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_FIRST_NAME, 
			      buffer ? buffer : "", NULL);      

      buffer = xmlGetProp(node, "MiddleName");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_MIDDLE_NAME, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "LastName");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_LAST_NAME, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Profession");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_PROFESSION, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "NamePrefix");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_NAME_PREFIX, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Title");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_TITLE, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Street");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_STREET, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "StreetNumber");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_STREET_NUMBER, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "ZipCode");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_ZIP_CODE, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "City");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CITY, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Province");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_PROVINCE, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "State");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_STATE, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Country");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_COUNTRY, 
			      buffer ? buffer : "", NULL);   
    }

  else if (strcmp(node->name, "Net") == 0)
    {
      buffer = xmlGetProp(node, "WebAddress");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_WEB_ADDRESS, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Web");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_WEB, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "EMailAddress");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_EMAIL_ADDRESS, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Email");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_EMAIL, 
			      buffer ? buffer : "", NULL);   
    }
  else if (strcmp(node->name, "Telephone") == 0)
    {
      buffer = xmlGetProp(node, "TelephoneNumbers");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_TELEPHONE_LIST, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Telephone");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_TELEPHONE_NUMBER, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "TelephoneType");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_TELEPHONE_TYPE, 
			      buffer ? buffer : "", NULL);         
    }
  else if (strcmp(node->name, "Work") == 0)
    {
      buffer = xmlGetProp(node, "CompanyName");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_COMPANY_NAME, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Operator");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_OPERATOR, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Fax");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_FAX, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Green");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_GREEN, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "CustomerCare");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CUSTOMER_CARE, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Assigment");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_ASSIGMENT, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Organization");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_ORGANIZATION, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Department");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_DEPARTMENT, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "SubDepartment");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_SUB_DEPARTMENT, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Secretary");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_SECRETARY, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "SecretaryPhone");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_SECRETARY_PHONE, 
			      buffer ? buffer : "", NULL); 
      }
  else if (strcmp(node->name, "Misc") == 0)
    {
      buffer = xmlGetProp(node, "SpouseName");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_SPOUSE_NAME, 
			      buffer ? buffer : "", NULL);   

      buffer = xmlGetProp(node, "Married");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_MARRIED, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Anniversary");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_ANNIVERSARY, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Children");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CHILDREN, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "ChildrenNotes");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CHILDREN_NOTES, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Hobbies");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_HOBBY, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "Notes");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_NOTES, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "PublicKey");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_PUBLIC_KEY, 
			      buffer ? buffer : "", NULL);

      buffer = xmlGetProp(node, "Birthday");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_BIRTHDAY, 
			      buffer ? buffer : "", NULL); 

      buffer = xmlGetProp(node, "CardName");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_CARD_NAME, 
			      buffer ? buffer : "", NULL);    

      buffer = xmlGetProp(node, "OtherNotes");
      gconf_client_set_string(client, RUBRICA_KEY_DICT_OTHER_NOTES, 
			      buffer ? buffer : "", NULL);
    }
  else
    return;
}

gboolean rubrica_dictionary_save_user_dictionary(GConfClient *client, 
						 gchar *language)
{
  xmlDocPtr doc;
  xmlNodePtr generale;
  xmlNodePtr personale;
  xmlNodePtr net;
  xmlNodePtr tel;
  xmlNodePtr work;
  xmlNodePtr misc;

  gchar *home, *home_dict, *dictionary;

  home       = rubrica_preferences_get_home();
  home_dict  = g_strdup_printf("%s/.rubrica/dictionary", home);
  dictionary = g_strdup_printf("%s/.rubrica/dictionary/%s", home, language);

  if (!g_file_exists(home_dict))
      mkdir(home_dict, S_IRUSR | S_IWUSR | S_IXUSR);

  doc = xmlNewDoc("1.0");

  doc->root = xmlNewDocNode(doc, NULL, "Rubrica_dictionary", NULL);
  xmlNodeSetLang(doc->root, language);
  xmlSetProp(doc->root, "version", VERSION);
  
  generale = xmlNewChild(doc->root, NULL, "General", "");
  xmlSetProp(generale, "Card", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CARD, NULL)); 
  xmlSetProp(generale, "Deleting", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_DELETE, NULL));
  xmlSetProp(generale, "Modifying", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_MODIFY, NULL));
  xmlSetProp(generale, "Created", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CREATED, NULL));
  xmlSetProp(generale, "LastChange", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_LAST_CHANGE, NULL));
  xmlSetProp(generale, "Data", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_DATA, NULL));
  xmlSetProp(generale, "Address", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_ADDRESS, NULL));
  xmlSetProp(generale, "Company", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_COMPANY, NULL));
  xmlSetProp(generale, "Work", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_WORK, NULL));

  personale = xmlNewChild(doc->root, NULL, "Personal", "");
  xmlSetProp(personale, "FirstName", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_FIRST_NAME, NULL));
  xmlSetProp(personale, "MiddleName", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_MIDDLE_NAME, NULL));
  xmlSetProp(personale, "LastName", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_LAST_NAME, NULL));
  xmlSetProp(personale, "Profession", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_PROFESSION, NULL));
  xmlSetProp(personale, "NamePrefix", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_NAME_PREFIX, NULL));
  xmlSetProp(personale, "Title", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_TITLE, NULL));

  xmlSetProp(personale,"Street", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_STREET, NULL));   
  xmlSetProp(personale, "StreetNumber", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_STREET_NUMBER, NULL));
  xmlSetProp(personale,"ZipCode", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_ZIP_CODE, NULL));
  xmlSetProp(personale,"City", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CITY, NULL));
  xmlSetProp(personale,"Province", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_PROVINCE, NULL));
  xmlSetProp(personale, "State", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_STATE, NULL));
  xmlSetProp(personale,"Country", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_COUNTRY, NULL));

  net = xmlNewChild(doc->root, NULL, "Net", "");
  xmlSetProp(net, "WebAddress", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_WEB_ADDRESS, NULL));
  xmlSetProp(net,"Web", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_WEB, NULL));   
  xmlSetProp(net, "EMailAddress", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_EMAIL_ADDRESS, NULL));
  xmlSetProp(net,"Email", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_EMAIL, NULL));

  tel = xmlNewChild(doc->root, NULL, "Telephone", "");
  xmlSetProp(tel, "TelephoneNumbers", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_TELEPHONE_LIST , NULL));
  xmlSetProp(tel, "Telephone", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_TELEPHONE_NUMBER, NULL));
  xmlSetProp(tel,"TelephoneType", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_TELEPHONE_TYPE, NULL));

  work = xmlNewChild(doc->root, NULL, "Work", "");
  xmlSetProp(work, "CompanyName", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_COMPANY_NAME, NULL));
  xmlSetProp(work, "Operator", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_OPERATOR, NULL));
  xmlSetProp(work, "Fax", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_FAX, NULL));
  xmlSetProp(work, "Green", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_GREEN, NULL));
  xmlSetProp(work, "CustomerCare", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CUSTOMER_CARE, NULL));
  xmlSetProp(work, "Assigment", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_ASSIGMENT, NULL));
  xmlSetProp(work, "Organization", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_ORGANIZATION, NULL));
  xmlSetProp(work, "Department", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_DEPARTMENT, NULL));
  xmlSetProp(work, "SubDepartment", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_SUB_DEPARTMENT, NULL));
  xmlSetProp(work, "Secretary", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_SECRETARY, NULL));
  xmlSetProp(work, "SecretaryPhone", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_SECRETARY_PHONE, NULL));

  misc = xmlNewChild(doc->root, NULL, "Misc", "");
  xmlSetProp(misc, "SpouseName", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_SPOUSE_NAME, NULL));
  xmlSetProp(misc, "Married", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_MARRIED, NULL));
  xmlSetProp(misc, "Anniversary", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_ANNIVERSARY, NULL));
  xmlSetProp(misc, "Children", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CHILDREN, NULL));
  xmlSetProp(misc, "ChildrenNotes", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CHILDREN_NOTES, NULL));
  xmlSetProp(misc, "Hobbies", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_HOBBY, NULL));
  xmlSetProp(misc, "Notes", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_NOTES, NULL));
  xmlSetProp(misc,"PublicKey",
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_PUBLIC_KEY, NULL));
  xmlSetProp(misc, "Birthday", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_BIRTHDAY, NULL));
  xmlSetProp(misc, "CardName", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_CARD_NAME, NULL));
  xmlSetProp(misc, "OtherNotes", 
	     gconf_client_get_string(client, RUBRICA_KEY_DICT_OTHER_NOTES, NULL));

  if( xmlSaveFile(dictionary, doc) == -1)
    {
      g_warning("can't save dictionary");
      
      return FALSE;
    }
  else
    g_print("\ndictionary %s saved in %s", dictionary, home_dict);
  
  xmlFreeDoc(doc);
  g_free(home_dict);
  g_free(dictionary);

  return TRUE;
}

/*  copy src into dest

void rubrica_dictionary_write(gchar *src, gchar **dest)
{
  if (src)
    *dest = g_strdup(src);
  else
    dest = NULL;     
}
 */
