/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>
#include <time.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"
#include "file.h"
#include "file_util.h"
#include "file_old.h"
#include "prefer.h"
#include "dictionary.h"
#include "addressbook.h"
#include "error.h"


gchar *doc_version;


void rubrica_file_salva(void)
{
  GtkWidget *app;
  gchar *nomefile;
  gchar *tmp;

  g_print("\n\nDebug log: enter rubrica_file_salva");
  app = rubrica_get_app();
  
  nomefile = rubrica_get_name();

  /* 
     se non c' un nome file di default allora bisogna richiamare salva_con_nome     
     if nomefile is NULL, then call rubrica_file_salva_come (ie save as)
  */
  if (!nomefile || g_strcasecmp(nomefile, _("no name")) == 0)
    {    
      /* salvo come default? */
      rubrica_file_salva_come(NULL, NULL);
    }
    /*   altrimenti provo a scrivere il file 
    */
  else 
    if (!rubrica_file_write_xmlfile(nomefile))
      {      
	tmp = g_strdup_printf(_("I can't save the file:\n%s"),nomefile);      
	gnome_app_error(GNOME_APP(app), tmp);
	g_free(tmp);
      }
  g_print("\nDebug log: exit rubrica_file_salva");
}

void 
rubrica_file_salva_come(GtkWidget *w, gpointer data)
{
  gchar *nomefile;
  GtkWidget *app;
  GtkFileSelection *fs;
  
  g_print("\n\nDebug log:enter rubrica_file_salva_come");

  app = rubrica_get_app();
  nomefile = rubrica_get_name();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Rubrica Save as")));

  if (rubrica_preferences_get_have_default_folder())
    {
      /* TODO 
	 
         settate la directory di default come corrente 
      */
    }
  
  /* se ho gi un nome di file lo mostro nell'entry */
  if (nomefile)
    gtk_file_selection_set_filename(fs, nomefile);
  
  /* connetto i bottoni ok e cancella */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_file_scrivi_file), fs);
  
  /* con il bottone cancella distruggo direttamente il widget */
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));

  g_print("\n\nDebug log: exit rubrica_file_salva_come");
}


void
rubrica_file_scrivi_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *tmp;
  gchar *fname = NULL;
  Rubrica *rub;
  gint page;          /* notebook's page that is active now */

  g_print("\n\nDebug log: enter rubrica_file_scrivi_file");

  page = rubrica_addressbook_get_current_page();
  rub = rubrica_addressbook_get_rubrica(page);

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));
  g_return_if_fail(rub != NULL);

  tmp = gtk_file_selection_get_filename(fs);

  /* se il file non ha estensione o l'estensione non  .rub */
  if (!rubrica_file_has_extension(tmp))
    fname = g_strdup_printf("%s.rub",tmp);  
  else 
    fname = g_strdup(tmp);
  
  /* se devo effettuare il backup */
  if (rubrica_preferences_get_do_backup())
    {
      gchar *newname;

      newname = g_strdup_printf("%s1", fname);
      rubrica_file_do_backup(fname, newname);
      g_free(newname);
    }

  /* se non  stato selezionato nessun file
     o non  possibile aprire il file segnalare l'errore */  
  if (!fname || !rubrica_file_write_xmlfile(fname)) 
    {
      gchar *err;
      GtkWidget *dlg;
      
      /* il dialogo di errore ha il widget fs come genitore */
      if (fname)
	err = g_strdup_printf(_("I can't save the file %s"), fname); 
      else
	err = g_strdup_printf(_("I can't save the file")); 
      
      dlg = gnome_error_dialog_parented(err, GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);   
      g_free(err);
    }
  else /* il file  stato salvato */
    {      
      gchar *path  = NULL;
      gchar *newname = NULL;

      rubrica_addressbook_set_flags(FALSE, FALSE, FALSE, page);
      
      if (rubrica_preferences_get_have_default_folder())  
	path  = rubrica_preferences_get_default_folder();
      else
	path = g_dirname(fname);
      newname = g_strdup_printf("%s/%s", path, g_basename(fname));
      
      rubrica_addressbook_set_path_and_name(path, g_basename(fname), page);
          
      /* imposta il file appena salvato come l'ultimo aperto/visualizzato 
      */
      rubrica_preferences_set_last_book(newname);
      
      rubrica_addressbook_make_this_rubrica_current(rub);

      gtk_widget_destroy(GTK_WIDGET(fs));
      
      g_free(newname);
      g_free(path);
    }

  g_free(fname);

  g_print("\nDebug log: exit rubrica_file_scrivi_file");
}

gboolean
rubrica_file_write_xmlfile(gchar *name)
{
  xmlDocPtr doc;  /* puntatore al documento xml */
  GList *rub;     /* ptr to rubrica, this contains data that we want save */
  GList *l;       /* alias to rub */
  gchar *path = NULL, *fname;
  
//  g_print("\n\nDebug log: enter rubrica_file_write_xmlfile");

  rub = rubrica_get_rubrica();

  g_return_val_if_fail(name != NULL, FALSE);
  g_return_val_if_fail(rub != NULL, FALSE);
  

  /* se l'utente vuole salvare le rubriche in una cartella predefinita 
     if users want to save addressbook in a default folder
  */
  if (rubrica_preferences_get_have_default_folder())  
    {
      name  = g_basename(name);
      path  = rubrica_preferences_get_default_folder();
      fname = g_strdup_printf("%s/%s", path, name);
    }
  else
    {
      path  = g_dirname(name);
      fname = g_strdup(name);
    }

  /* creazione di un nuovo documento xml */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica */
  doc->root = xmlNewDocNode(doc, NULL, "Rubrica", NULL);
  xmlSetProp(doc->root, "version", VERSION);

  /* attraverso la rubrica e scrivo i dati 
     nell'albero xml
  */
  for (l = rub; l != NULL; l = l->next) 
    {
      RubricaItem *item = l->data;
      
      rubrica_file_build_xmlnode(doc->root, item);
    }
  
  /* verifico che non esista un file con lo stesso nome dell'attuale
     se chiesto, faccio il backup del vecchio
     altrimenti cancello il vecchio file e scrivo il nuovo 
  */
  if (g_file_exists(fname))
    {
      if (rubrica_preferences_get_do_backup())
	rubrica_file_do_backup(fname, fname);
      else
	remove(fname);
    }
  
  if (rubrica_preferences_get_compress)
    {
      xmlSetCompressMode(rubrica_preferences_get_compress_rate());
      xmlSetDocCompressMode(doc, rubrica_preferences_get_compress_rate());
    }
  else
    {
      xmlSetDocCompressMode(doc, 0);     
      xmlSetCompressMode(0);
    }
  
  if (xmlSaveFile(fname, doc) == -1) 
    {
      g_print("\n\tError Can't save the file: %s", fname);

      xmlFreeDoc(doc);
      return FALSE;
    }  

  /* if we are here the addressbook is write */
  g_print("\naddressbook saved as: %s in folder: %s\n", name, path);
  rubrica_view_update_path(path);

  /* xmlFreeDoc do a segfault, why? */
  //  xmlFreeDoc(doc);    
  g_free(fname);

//  g_print("\nDebug log: exit rubrica_file_write_xmlfile");
  return TRUE;
}

void 
rubrica_file_build_xmlnode(xmlNodePtr xmlnode, RubricaItem *item)
{
  GList *alias;
  xmlNodePtr cardxml;

  xmlNodePtr datixml;
  xmlNodePtr addressxml;
  xmlNodePtr webxml;
  xmlNodePtr mailxml;
  xmlNodePtr telxml;
  xmlNodePtr companyxml;
  xmlNodePtr workxml;
  xmlNodePtr notexml;
  gchar *str;

  xmlNodePtr nodo;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);

  g_print("\n\nDebug log: enter rubrica_file_build_xmlnode");

  /*
    cardxml  figlio di xmlnode (il primo xmlnode  doc->root)
    schedaxml, datixml, ..., sono figli di cardxml
  */    
  cardxml = xmlNewChild(xmlnode, NULL, "Card", NULL);

  /* setto le propriet della scheda 
     nome, editabile, concellabile
  */
  xmlSetProp(cardxml, "name",  item->scheda ? item->scheda : "");
  xmlSetProp(cardxml, "modifying", item->editabile    ? "true" : "false");
  xmlSetProp(cardxml, "deleting",  item->cancellabile ? "true" : "false");

  /* data di creazione e ultima modifica della scheda */
  str = g_strdup_printf("%ld", item->created);
  xmlSetProp(cardxml, "created", str);
  g_free(str);
  str = g_strdup_printf("%ld", item->last_change);  
  xmlSetProp(cardxml, "last_change", str);
  g_free(str);

  /*
    xmlNewTextChild 
    sintassi :
         - nodo genitore del nodo creato con la chiamata
	 - namespace
	 - nome del nodo creato
	 - testo (elemento) da inserire nel nodo creato
  */
  
  /* ho creato l'elemento xml es.: 
     <Scheda created="1003016840" last_change="1003016840" editabile="true" 
      cancellabile="FALSE>
           scheda numero uno
     </Scheda>

     <Card created="1003016840" last_change="1003016840" modifying="true" 
      deleting="false">
           card number one
     </Card>
  */

  /* Personal */
  datixml = xmlNewTextChild(cardxml, NULL, "Data", NULL);
  nodo = xmlNewTextChild(datixml, NULL, "FirstName", item->personal.first_name);
  xmlSetProp(nodo, "know_birthday", (item->personal.birthday_setted 
				     ? "true" : "false"));
  str = g_strdup_printf("%ld", item->personal.birthday);
  xmlSetProp(nodo, "birthday", str);
  g_free(str);  
  nodo = xmlNewTextChild(datixml, NULL, "MiddleName", item->personal.middle_name);
  nodo = xmlNewTextChild(datixml, NULL, "LastName", item->personal.last_name);
  nodo = xmlNewTextChild(datixml, NULL, "Profession", item->personal.profession);
  nodo = xmlNewTextChild(datixml, NULL, "NamePrefix", item->personal.name_prefix);
  nodo = xmlNewTextChild(datixml, NULL, "Title", item->personal.title);

  /* Address */
  addressxml = xmlNewTextChild(cardxml, NULL, "Address", NULL);
  nodo = xmlNewTextChild(addressxml, NULL, "Street", item->address.street);
  nodo = xmlNewTextChild(addressxml, NULL, "StreetNumber", item->address.number);
  nodo = xmlNewTextChild(addressxml, NULL, "ZipCode", item->address.zip_code);
  nodo = xmlNewTextChild(addressxml, NULL, "City", item->address.city);
  nodo = xmlNewTextChild(addressxml, NULL, "Province", item->address.province);
  nodo = xmlNewTextChild(addressxml, NULL, "State", item->address.state);
  nodo = xmlNewTextChild(addressxml, NULL, "Country", item->address.country);

  /* Net */
  webxml = xmlNewChild(cardxml, NULL, "WebAddresses", NULL);
  alias = item->net.web_list;
  while(alias)
    {
      nodo = xmlNewTextChild(webxml, NULL, "Web", alias->data);

      alias = alias->next;
    }
  
  mailxml = xmlNewChild(cardxml, NULL, "EmailAddresses", NULL);
  alias = item->net.mail_list;
  while(alias)
    {
      nodo = xmlNewTextChild(mailxml, NULL, "Email", alias->data);
  
      alias = alias->next;
    }

  
  /* Telephone */
  telxml = xmlNewChild(cardxml, NULL, "TelephoneNumbers", NULL);
  alias = item->telephone.tel_list;
  while(alias)
    {
      TelNum *tt = (TelNum*) alias->data;

      nodo = xmlNewTextChild(telxml, NULL, "Telephone", tt->numero_telefono);      
      xmlSetProp(nodo, "type", tt->tipo);
      
      alias = alias->next;
    }

  /* Company */
  companyxml = xmlNewTextChild(cardxml, NULL, "Company", NULL);
  nodo = xmlNewTextChild(companyxml, NULL, "CompanyName", item->company.name);
  nodo = xmlNewTextChild(companyxml, NULL, "Street", item->company.street);
  nodo = xmlNewTextChild(companyxml, NULL, "StreetNumber", item->company.number);
  nodo = xmlNewTextChild(companyxml, NULL, "ZipCode", item->company.zip_code);
  nodo = xmlNewTextChild(companyxml, NULL, "City", item->company.city);
  nodo = xmlNewTextChild(companyxml, NULL, "Province", item->company.province);
  nodo = xmlNewTextChild(companyxml, NULL, "Country", item->company.country);
  nodo = xmlNewTextChild(companyxml, NULL, "Web", item->company.web);
  nodo = xmlNewTextChild(companyxml, NULL, "Email", item->company.email);
  nodo = xmlNewTextChild(companyxml, NULL, "Operator", item->company.operator);
  nodo = xmlNewTextChild(companyxml, NULL, "Fax", item->company.fax);
  nodo = xmlNewTextChild(companyxml, NULL, "Green", item->company.green);
  nodo = xmlNewTextChild(companyxml, NULL, "CustomerCare", 
			 item->company.customer_care);
  nodo = xmlNewTextChild(companyxml, NULL, "Notes", item->company.notes);
  
  
  /* Work */
  workxml = xmlNewTextChild(cardxml, NULL, "Work", NULL);
  nodo = xmlNewTextChild(workxml, NULL, "Assigment", item->work.assigment);
  nodo = xmlNewTextChild(workxml, NULL, "Organization", item->work.organization);
  nodo = xmlNewTextChild(workxml, NULL, "Department", item->work.department);
  nodo = xmlNewTextChild(workxml, NULL, "SubDepartment", item->work.sub_dep);
  nodo = xmlNewTextChild(workxml, NULL, "SecretaryName", item->work.secretary_name);
  nodo = xmlNewTextChild(workxml, NULL, "SecretaryPhone", item->work.secretary_phone);


  /* Notes */
  notexml = xmlNewTextChild(cardxml, NULL, "Notes", NULL);
  xmlSetProp(notexml, "married",  item->notes.married  ? "true" : "false");
  xmlSetProp(notexml, "children", item->notes.children ? "true" : "false");  

  nodo = xmlNewTextChild(notexml, NULL, "SpouseName", item->notes.spouse_name);
  str = g_strdup_printf("%ld", item->notes.birthday);
  xmlSetProp(nodo, "birthday",  str);
  g_free(str);
  str = g_strdup_printf("%ld", item->notes.anniversary);
  xmlSetProp(nodo, "anniversary",  str);
  g_free(str);  

  nodo = xmlNewTextChild(notexml, NULL, "Child", item->notes.child);
  nodo = xmlNewTextChild(notexml, NULL, "Hobbies", item->notes.hobby);
  nodo = xmlNewTextChild(notexml, NULL, "OtherNotes", item->notes.notes);
  nodo = xmlNewTextChild(notexml, NULL, "PublicKey", item->notes.public_key);

  g_print("\nDebug log: exit rubrica_file_build_xmlnode");
}



/**
 **
 **  Load Routines
 **
 **/
/*  this function is called when user want start rubrica whit a file
*/
void rubrica_file_start_with_file(gchar *file)
{
  Rubrica *rub;
  gboolean loaded;
  RUBRICA_ERROR err;

  rubrica_file_set_start(TRUE);

  /* load file */
  loaded = rubrica_file_load_xmlfile(file, &err);
  if (!loaded)
    {
      rubrica_error_display_error(NULL, file, err);      
    }
  
  rubrica_file_set_start(FALSE);
  rub = rubrica_addressbook_get_rubrica(0);
  rubrica_addressbook_make_this_rubrica_current(rub);
}

void 
rubrica_file_carica_file(void)
{
  GtkWidget *fs;
  GtkWidget *app;

  app = rubrica_get_app();

  fs = gtk_file_selection_new(_("Open file..."));

  /* connetto i bottoni ok e cancel */
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_file_apri_file), fs);
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button), 
			    "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  /* visualizzo la finestra in posizione centrale */
  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection  app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}

void 
rubrica_file_apri_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *filename;
  gboolean loaded;
  RUBRICA_ERROR err;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  filename = gtk_file_selection_get_filename(fs);

  loaded = rubrica_file_load_xmlfile(filename, &err);
  if (!loaded)
    rubrica_error_display_error(GTK_WIDGET(fs), filename, err);
  else
    gtk_widget_destroy(GTK_WIDGET(fs));    
}


gboolean rubrica_file_load_xmlfile(gchar *filename, RUBRICA_ERROR* err)
{
  gint page = 0;

  if (!g_file_exists(filename))
    {
      *err = RUBRICA_ERROR_FILE_NOT_EXIST;
            
      return FALSE;
    }

  page = rubrica_addressbook_add_page(filename); 

  /* real file load */
  if (!rubrica_file_real_load_xmlfile(filename, err))
    return FALSE;
  else 
    {
      /* if we are here, the file has been loaded, 
	 now we need to put in memory some information 
	 about this file and addressbook      
      */
      gchar *fpath, *fname;

      rubrica_set_empty(FALSE);      
      fpath = g_dirname(filename);
      fname = g_basename(filename);
      
      rubrica_addressbook_set_flags(FALSE, FALSE,FALSE, page);
      rubrica_addressbook_set_path_and_name(fpath, fname, page); 
      rubrica_addressbook_make_this_page_current(page);
    }  

  *err = RUBRICA_ERROR_NO_ERROR;
  return TRUE;
}


/* 
 *  read the addressbook file 
 */
gboolean 
rubrica_file_real_load_xmlfile(gchar *fname, RUBRICA_ERROR *err)
{
  xmlDocPtr doc;

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* read and parse file */
  doc = xmlParseFile(fname);

  if (!doc)
    {
      *err = RUBRICA_ERROR_PARSE_XML_DOC;
      return FALSE;
    }

  if (!doc->root || 
      !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      *err = RUBRICA_ERROR_XML_DOC_NO_RUBRICA_HEADER;
      
      xmlFreeDoc(doc);
      return FALSE;
    }

  /* parse doc, extract address from file and put them in memory */
  rubrica_file_parse_doc(doc, fname, err);

  /* addressbook has been parsed. Free memory */
  g_print("\nRubrica doc is parsed.");
  xmlFreeDoc(doc);
 
  return TRUE;  
}


void 
rubrica_file_parse_doc(xmlDocPtr doc, gchar *fname, RUBRICA_ERROR *err)
{  
  xmlNodePtr nodo;
  gint doc_ver = 0;

  doc_version = rubrica_file_get_version(doc);

  doc_ver = rubrica_file_str2int(doc_version);

  g_print("\nloading file made by rubrica version: %s\n", 
	  doc_version ? doc_version : "< 0.8");
  /* 	carico il vecchio file 
   */  
  if ((doc_version == NULL) || (doc_ver < 7.9))
    {
      *err = rubrica_file_old_parse_rubrica_doc(doc);

      return;
    }
  
  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)    
    rubrica_file_load_from_xmlnode(nodo);  

  *err = RUBRICA_ERROR_NO_ERROR;
}

void
rubrica_file_load_from_xmlnode(xmlNodePtr xmlnode)
{
  RubricaItem *r;  
  GList *web, *mail, *tel;
  xmlNodePtr figlio;
  /* i nodi xmlnode sono nodi cardxml
     - devo estrarre i figli di cardxml:
  */

  r = g_malloc(sizeof(RubricaItem));

  /* Card */
  if (xmlnode &&
      xmlnode->name &&                            /* se il nodo ha nome e */
      g_strcasecmp(xmlnode->name, "Card") == 0)   /* il nome  Scheda     */
    {
      gchar *tmp;  

      r->scheda = ((tmp = xmlGetProp(xmlnode, "name")) ? g_strdup(tmp) : "");
      if (tmp)
	g_free(tmp);
 
      r->editabile = ((g_strcasecmp(xmlGetProp(xmlnode, "modifying"), 
				    "true") == 0) ? TRUE : FALSE);
      r->cancellabile = ((g_strcasecmp(xmlGetProp(xmlnode, "deleting"), 
				       "true") == 0) ? TRUE : FALSE);
      
      r->created = ((tmp = xmlGetProp(xmlnode, "created")) ? 
		    atol(tmp) : time(NULL));
      if (tmp)
	g_free(tmp);
      
      r->last_change = ((tmp = xmlGetProp(xmlnode, "last_change")) ? 
			atol(tmp) : time(NULL));
      if (tmp)
	g_free(tmp);      
    }
  else
    {
      /* TO DO 
	 
      segnalare l'errore, far scegliere all'utente se 
      continuare a caricare automaticamente il file 
      o vuole un caricamento assistito
      */	 
    }
  
  /* dati personali -- datixml -- */
  figlio = xmlnode->childs;
  if (figlio &&
      figlio->name &&
      g_strcasecmp(figlio->name, "Data") == 0)  
    {
      /* carico i figli di datixml
       */
      xmlNodePtr nodo;
      gchar *tmp;
      
      nodo = figlio->childs;
      r->personal.first_name = ((tmp = xmlNodeGetContent(nodo)) ? 
				g_strdup(tmp) : ""); 

      if ((tmp = xmlGetProp(nodo, "know_birthday")))
	r->personal.birthday_setted = ((g_strcasecmp(tmp, "true") == 0) ?
				       TRUE : FALSE);
      else
	r->personal.birthday_setted = FALSE;	  
     
      r->personal.birthday = ((tmp = xmlGetProp(nodo, "birthday")) ? 
			      atol(tmp) : time(NULL));
      
      nodo = nodo->next;
      r->personal.middle_name = ((tmp = xmlNodeGetContent(nodo)) ? 
				 g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->personal.last_name = ((tmp  = xmlNodeGetContent(nodo)) ?
			       g_strdup(tmp) : "");
      
      nodo = nodo->next;      
      r->personal.profession = ((tmp = xmlNodeGetContent(nodo)) ? 
				g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->personal.name_prefix = ((tmp = xmlNodeGetContent(nodo)) ? 
				 g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->personal.title = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
    }
  else 
    {
      g_warning(_("May be this file is corrupted. Can't read Data field"));
      
      r->personal.first_name = r->personal.middle_name 
	= r->personal.last_name = r->personal.profession 
	= r->personal.name_prefix = r->personal.title = "";
      r->personal.birthday = time(NULL);
    }
  
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      g_strcasecmp(figlio->name, "Address") == 0)
    {
      xmlNodePtr nodo;
      gchar *tmp;
      
      nodo = figlio->childs;  
      r->address.street = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->address.number = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->address.zip_code = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->address.city = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->address.province = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->address.state = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      
      nodo = nodo->next;
      r->address.country = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
    }
  else 
    {
      g_warning(_("May be this file is corrupted. Can't read Address field"));
      
      r->address.street = r->address.number = r->address.zip_code = 
	r->address.city = r->address.province = r->address.state = 
	r->address.country = "";
    }
  
  /* web */
  web = NULL;
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      g_strcasecmp(figlio->name, "WebAddresses") == 0)
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      web = g_list_append(web, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->net.web_list = web;
      r->net.nweb = n;
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read WebAddresses field"));
      
      r->net.web_list = NULL;
      r->net.nweb = 0;
    }

  /* email */
  figlio = figlio->next;
  mail = NULL;
  if (figlio &&
      figlio->name &&
      g_strcasecmp(figlio->name, "EmailAddresses") == 0)
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      mail = g_list_append(mail, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->net.mail_list = mail;
      r->net.nmail = n;
    }  
  else
    {
      g_warning(_("May be this file is corrupted. Can't read EmailAddresses field"));
      
      r->net.mail_list = NULL;
      r->net.nmail = 0;
    }
  
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      g_strcasecmp(figlio->name, "TelephoneNumbers") == 0)
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp1=NULL, *tmp2=NULL;
	  
	  tmp1 = xmlNodeGetContent(nodo);
	  if ((strcmp(nodo->properties->name, "type") == 0))
	    tmp2 = xmlGetProp(nodo, "type");
	  else 
	    tmp2 = "???";

	  if (tmp1)
	    {	
	      t = g_malloc(sizeof(TelNum));
	      
	      t->numero_telefono = g_strdup(tmp1);
	      t->tipo = g_strdup(tmp2);	  	     
	      
	      tel = g_list_append(tel, t);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->telephone.tel_list = tel;
      r->telephone.ntel = n;
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read TelephoneNumbers field"));
      
      r->telephone.tel_list = NULL;
      r->telephone.ntel = 0;
    }
  
  /* company */
  figlio = figlio->next; 
  if (figlio &&
      figlio->name && 
      g_strcasecmp(figlio->name, "Company") == 0)
    {
      xmlNodePtr nodo;
      gchar *tmp;
      
      nodo = figlio->childs;
      r->company.name = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.street = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.number = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.zip_code = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.city = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.province = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.country = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.web = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.email = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.operator = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.fax = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.green = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.customer_care =((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->company.notes = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
    }
  else   
    {
      g_warning(_("May be this file is corrupted. Can't read Company field"));
      
      r->company.name = r->company.street = r->company.number 
	= r->company.zip_code = r->company.city  = r->company.province
	= r->company.country = r->company.web = r->company.email
	= r->company.operator = r->company.fax = r->company.green 
	= r->company.customer_care = r->company.notes = "";
    }

  /* work */
  figlio = figlio->next; 
  if (figlio &&
      figlio->name && 
      g_strcasecmp(figlio->name, "Work") == 0)
    {
      xmlNodePtr nodo;
      gchar *tmp;
      
      nodo = figlio->childs;
      r->work.assigment = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->work.organization = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->work.department = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->work.sub_dep = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->work.secretary_name = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->work.secretary_phone = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
       
    }
  else
    {
      g_warning(_("May be this file is corrupted. Can't read Work field"));

      r->work.assigment =  r->work.organization = 
	r->work.department = r->work.sub_dep = 
	r->work.secretary_name = r->work.secretary_phone = "";
    }

  /* note */
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      g_strcasecmp(figlio->name, "Notes") == 0)
    {
      xmlNodePtr nodo;
      gchar *tmp;
      
      tmp = xmlGetProp(figlio, "married");
      r->notes.married = ((g_strcasecmp(tmp, "true") == 0) ? TRUE : FALSE);

      tmp = xmlGetProp(figlio, "children");
      r->notes.children = ((g_strcasecmp(tmp, "true") == 0) ? TRUE : FALSE);
      
      nodo = figlio->childs;
      r->notes.spouse_name = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
      r->notes.birthday = ((tmp = xmlGetProp(nodo, "birthday")) ? 
			   atol(tmp) : time(NULL));      
      r->notes.anniversary = ((tmp = xmlGetProp(nodo, "anniversary")) ? 
			      atol(tmp) : time(NULL));        

      nodo = nodo->next;
      r->notes.child = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->notes.hobby = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->notes.notes = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");

      nodo = nodo->next;
      r->notes.public_key = ((tmp = xmlNodeGetContent(nodo)) ? g_strdup(tmp) : "");
    } 
  else
    {
      g_warning(_("May be this file is corrupted. Can't read Notes field"));

      r->notes.spouse_name = r->notes.hobby = 
	r->notes.notes = r->notes.public_key = "";
      r->notes.married = r->notes.children = FALSE;
      r->notes.birthday = r->notes.anniversary = time(NULL);
    }

  rubrica_inserisci_elemento(r);
  rubrica_view_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}



