/*
*  Rubrica
*  file: file_old.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

// #include <stdlib.h>
// #include <ctype.h>
// #include <math.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "rubrica.h"
#include "main.h"
// #include "addressbook.h"
#include "file_old.h"
#include "error.h"

/* 
 *                                                                   
 *     routine di compatibilit. Leggono le rubriche create            
 *     con versioni precedenti alla 0.8                               
 *                                                                     
 *     Routine for backward compatibility. Reads old rubrica files     
 *                                                                    
 */
RUBRICA_ERROR 
rubrica_file_old_parse_rubrica_doc(xmlDocPtr doc)
{
  xmlNodePtr nodo;
  RUBRICA_ERROR err = RUBRICA_ERROR_NO_ERROR;

  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)
    {

      err = rubrica_file_old_load_element_from_xml(nodo); 

      if (err != RUBRICA_ERROR_NO_ERROR)
	return err;
    }
   
  return (RUBRICA_ERROR) err;
}


RUBRICA_ERROR
rubrica_file_old_load_element_from_xml(xmlNodePtr xmlnode)
{
  RubricaItem *r;  
  GList *web, *mail, *tel;
  xmlNodePtr figlio;
  gchar *buffer;

  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */
  /* se il nodo non ha nome o il nome non  Indirizzo  */    
  if (xmlnode->name == NULL)                     
    return RUBRICA_ERROR_XML_DOC_NO_RUBRICA_FIELD;
  
  if (g_strcasecmp(xmlnode->name, "Address") != 0)
    if (g_strcasecmp(xmlnode->name, "Indirizzo") != 0) 
      return RUBRICA_ERROR_XML_DOC_NO_RUBRICA_FIELD;
  
  r = g_malloc(sizeof(RubricaItem));

  /* schedaxml */
  figlio = xmlnode->childs;
  if (figlio &&
      figlio->name && 
      ( (g_strcasecmp(figlio->name, "Card") == 0) ||
	(g_strcasecmp(figlio->name, "Scheda") == 0)))
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
  else     
      r->scheda = g_strdup("");
      
  /* scheda ha degli attributi, bisogna estrarli */
  if ((buffer = xmlGetProp(figlio, "modifying")) == NULL)
    buffer = xmlGetProp(figlio, "editabile");
  if (buffer)
    {
      r->editabile = (g_strcasecmp(buffer, "TRUE") == 0) ? TRUE : FALSE;
      g_free(buffer);
    }
  else r->editabile = TRUE;

  if ((buffer = xmlGetProp(figlio, "deleting")) == NULL)
    buffer = xmlGetProp(figlio, "cancellabile");
  if (buffer)
    {
      r->cancellabile = (g_strcasecmp(buffer, "TRUE") == 0) ? TRUE : FALSE;
      g_free(buffer);
    }
  else r->cancellabile = TRUE;
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (figlio &&
      figlio->name &&
      ( (g_strcasecmp(figlio->name, "Data") == 0) ||
        (g_strcasecmp(figlio->name, "Dati") == 0)))
    {
      /* carico i figli di datixml
	   - datixml=figlio 
	   - nodo = figlio di datixml
      */
      xmlNodePtr nodo;

      r->personal.middle_name = "";
      r->personal.name_prefix = "";
      r->personal.title = "";
      r->personal.birthday_setted = FALSE;
      r->personal.birthday = time(NULL);	
      r->address.state = "";

      nodo = figlio->childs;
      r->personal.first_name = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->personal.last_name = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->personal.profession = g_strdup(xmlNodeGetContent(nodo));
 
      nodo = nodo->next;
      r->address.street = g_strdup(xmlNodeGetContent(nodo));
      
      nodo = nodo->next;
      r->address.number = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->address.zip_code = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->address.city = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->address.province = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->address.country = g_strdup(xmlNodeGetContent(nodo));
    }
  else 
    {
      r->personal.first_name = r->personal.middle_name 
	= r->personal.last_name = r->personal.profession 
	= r->personal.name_prefix = r->personal.title = "";
      r->personal.birthday = time(NULL);    
    }

  /* web */
  web = NULL;
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      ( (g_strcasecmp(figlio->name, "WebAddresses") == 0) ||
	(g_strcasecmp(figlio->name, "Indirizzi_Web") == 0)))
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));
	  
	  web = g_list_append(web, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->net.web_list = web;
      r->net.nweb = n;
    }
  else
    {
      r->net.web_list = NULL;
      r->net.nweb = 0;
    }
  
  /* mail */
  figlio = figlio->next;
  mail = NULL;
  if (figlio &&
      figlio->name &&
      ( (g_strcasecmp(figlio->name, "EmailAddresses") == 0) ||
	(g_strcasecmp(figlio->name, "Indirizzi_Email") == 0)))
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;

      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));

	  mail = g_list_append(mail, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->net.mail_list = mail;
      r->net.nmail = n;
    }  
  else
    {
      r->net.mail_list = NULL;
      r->net.nmail = 0;
    }
    
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      ( (g_strcasecmp(figlio->name, "TelephoneNumbers") == 0) ||
	(g_strcasecmp(figlio->name, "Numeri_Telefonici") == 0)))
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tipo;

	  t = g_malloc(sizeof(TelNum));

	  tipo = xmlGetProp(nodo,"Type");
	  if (!tipo)
	    tipo =  xmlGetProp(nodo,"Tipo");
	  if (!tipo)
	    tipo = xmlGetProp(nodo,"type");
	  if (!tipo)
	    tipo = xmlGetProp(nodo,"tipo");

	  t->numero_telefono = g_strdup(xmlNodeGetContent(nodo));
	  t->tipo = g_strdup(tipo);
	  
	  tel = g_list_append(tel, t);
	  nodo = nodo->next;
	  n++;
	}
      r->telephone.tel_list = tel;
      r->telephone.ntel = n;
    }
  else
    {
      r->telephone.tel_list = NULL;
      r->telephone.ntel = 0;
    }
  
  r->company.name = r->company.street = r->company.number 
    = r->company.zip_code = r->company.city  = r->company.province
    = r->company.country = r->company.web = r->company.email
    = r->company.operator = r->company.fax = r->company.green 
    = r->company.customer_care = r->company.notes = "";
  
  r->work.assigment =  r->work.organization = 
    r->work.department = r->work.sub_dep = 
    r->work.secretary_name = r->work.secretary_phone = "";

  r->notes.married  = r->notes.children = FALSE;
  r->notes.birthday = r->notes.anniversary = time(NULL);
  r->notes.spouse_name =r->notes.child = 
    r->notes.hobby = r->notes.notes = 
    r->notes.public_key = "";

  /* note */
  figlio = figlio->next;
  if (figlio &&
      figlio->name &&
      ( (g_strcasecmp(figlio->name, "Notes") == 0) || 
	(g_strcasecmp(figlio->name, "Note") == 0)))
    r->notes.notes = g_strdup(xmlNodeGetContent(figlio));
  else 
    r->notes.notes = g_strdup("");
  
  rubrica_inserisci_elemento(r);
  rubrica_view_inc_indirizzi();
  rubrica_inserisci_scheda(r);

  return RUBRICA_ERROR_NO_ERROR;
}
