/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <math.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "file_util.h"

gboolean start = FALSE;


gchar *rubrica_file_get_version(xmlDocPtr doc)
{
  if ((doc->root->properties)       &&
      (doc->root->properties->name) &&
      strcmp(doc->root->properties->name, "version") == 0)
    return  xmlGetProp(doc->root, "version");

  return NULL;
}


gboolean rubrica_file_is_start(void)
{
  return start;
}

void rubrica_file_set_start(gboolean state)
{
  start = state;
}

/*  return:
    TRUE  if filename has extension .rub
    FALSE otherwise
*/
gboolean rubrica_file_has_extension(gchar *fn)
{
  gchar *tmp=NULL, *buf;
  gint i, len;

  i = 0;
  buf = g_basename(fn);
  len = strlen(buf);

  while(*buf != '.')
    {
      buf++;
      i++;
            
      if (i == len)     /* end buffer was reached and dot wasn't found */
	return FALSE;	
    }

  tmp = (gchar*) g_malloc(((len - i)*sizeof(gchar))+1);
  strcpy(tmp, buf);
  
  if ((strcmp(tmp,".rub")) == 0)
    {
      g_free(tmp);
      return TRUE;
    }
  else 
    g_free(tmp);
  return FALSE;  
}

void rubrica_file_do_backup(gchar *old, gchar *new)
{
  gint err;

  /* se esiste gi un file con lo stesso nome, crea una copia di backup 
     if exist a file with same name, then do a backup of old file */
  if (g_file_exists(new))
    {
      gchar *path;
      gchar *ext=NULL, *f=NULL;
      gchar *bfile;
      gint num;

      path = g_dirname(new);
      rubrica_file_split(new, &f, &ext, &num);

      bfile = g_strdup_printf("%s/%s%d.%s", path, f, num, ext);
      rubrica_file_do_backup(old, bfile);
      g_free(bfile);
    }
  else
    {
      err = rename(old, new);  
      
      if (!err)
	g_print("\nBackup -- old file: %s has been saved in: %s", old, new);
      else
	g_print("\nBackup -- This error occurred during backup: %s", strerror(errno)); 
    }
}

void rubrica_file_split(gchar *file, gchar **f, gchar **ext, gint *n)
{
  gchar *buf, *tmp, **split;
  gint i = 0, num = 0;

  buf = g_basename(file);
  split = g_strsplit(buf, ".", 1);

  *ext = g_strdup(split[1]);

  buf = split[0];
  g_strreverse(buf);

  while (isdigit(buf[i]))
    {
      num = num + (buf[i] - 48) * pow(10,i);
      i++;
    }
  num+=1;
  g_strreverse(buf);
  tmp = g_strndup(split[0], strlen(split[0]) - i);
  
  *f = tmp;
  *n = num;
}


int rubrica_file_str2int(gchar *str)
{
  int len;
  int ret = 0;
  int i = 0;
  
  len = strlen(str)-1;

  while (str[len])
    {
      if (isdigit(str[len]))
	{
	  ret = ret + (str[len] - '0') * pow(10, i);
	  i++;
	}
      len--;
    }

  return ret;
}
