/*
*  Rubrica
*  file: import.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "file.h"
#include "file_old.h"
#include "main.h"
#include "error.h"
#include "import.h"
#include "import_int.h"
#include "import_gcard.h"
#include "addressbook.h"

ImportFrom importfrom;

static gchar *gcard_default="/.gnome/GnomeCard.gcrd";
static gchar *card_kde1="/.kde/share/apps/kab/addressbook.database";
static gchar *card_kde2="/.kde/share/apps/kab/";


void rubrica_import_init_item(RubricaItem *r)
{
  r->editabile = r->cancellabile = TRUE;      
  r->scheda = "";

  r->personal.first_name = r->personal.middle_name 
    = r->personal.last_name = r->personal.profession 
    = r->personal.name_prefix = r->personal.title = NULL;
  r->personal.birthday = time(NULL);
      
  r->address.street = r->address.number = r->address.zip_code = 
    r->address.city = r->address.province = r->address.state = 
    r->address.country = NULL;

  r->net.web_list = r->net.mail_list = r->telephone.tel_list = NULL;
  r->net.nweb = r->net.nmail = r->telephone.ntel = 0; 

  r->company.name = r->company.street = r->company.number 
    = r->company.zip_code = r->company.city  = r->company.province
    = r->company.country = r->company.web = r->company.email
    = r->company.operator = r->company.fax = r->company.green 
    = r->company.customer_care = r->company.notes = NULL;
  
  r->work.assigment =  r->work.organization = 
    r->work.department = r->work.sub_dep = 
    r->work.secretary_name = r->work.secretary_phone = NULL;
  
  r->notes.married = r->notes.children = FALSE;
  r->notes.birthday = r->notes.anniversary = time(NULL);
  r->notes.spouse_name = r->notes.child = r->notes.hobby = 
    r->notes.notes = r->notes.public_key = NULL;

}

void rubrica_import_from_init(void)
{
  importfrom = GCARD; /* import default will be csv file */
}

void rubrica_import_from_set(ImportFrom i)
{
  importfrom = (ImportFrom) i;
}

ImportFrom rubrica_import_from_get(void)
{
  return (ImportFrom) importfrom;
}


void rubrica_import_data(void)
{
  RubricaAddressBook *adb;
  GtkWidget *fentry, *app;
  gchar *filename;
  gchar *buffer, *titolo = NULL;
  gint n, page;
  
  n=0;
  app = rubrica_get_app();
  adb = rubrica_get_addressbook();
  fentry = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY
				      (rubrica_import_get_file_entry()));

  filename = gtk_entry_get_text(GTK_ENTRY(fentry));

  page = rubrica_addressbook_add_page(NULL);
  
  switch (importfrom)
    {
    case GCARD:
      /* 	 if no name is set in gnome_entry, then 
		 load default gnomecard addressbook 
      */
      if (strcmp(filename,"") == 0)
	{
	  /* carico il file di default */
	  buffer = g_get_home_dir();
	  
	  filename = g_strdup_printf("%s%s",buffer,gcard_default);
	  rubrica_import_open_file(filename);
	  titolo = _("Imported from GnomeCard");
	  n = rubrica_view_get_indirizzi();
	}          
      else	
	/* import gnomecard file */
	{
	  rubrica_import_open_file(filename);
	  n = rubrica_view_get_indirizzi();		  
	}
      break;
      
    case KCARD:
      /* kab (kab 2.x) has a default database */
      buffer =  g_get_home_dir();
      
      filename = g_strdup_printf("%s%s",buffer,card_kde1);
      rubrica_import_open_file(filename);
      titolo = _("Imported from Kab");
      n = rubrica_view_get_indirizzi();
      break;

    case RUBRICA:
      if (strcmp(filename, "") == 0)
	{
	  GtkWidget *app;
	  
	  app = rubrica_get_app();  
	  gnome_app_error(GNOME_APP(app), _("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else
      /* apri il file e leggilo */
	{
	  g_print("\nimport %s", filename);
	  n = rubrica_import_open_old_rubrica_file(filename);      
	}
      break;

    case CSV:
    default:
      if (strcmp(filename,"") == 0)
	{
	  GtkWidget *app;
	  
	  app = rubrica_get_app();  
	  gnome_app_error(GNOME_APP(app), _("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else 
	{
	  g_print("file: %s", filename);
	  rubrica_import_open_file(filename);      
	  titolo = _("Imported from Netscape csv");	  
	  n = rubrica_view_get_indirizzi();
	}
      break;      
    }

  rubrica_addressbook_set_path_and_name(g_dirname(filename),g_basename(filename), 
					page);
  rubrica_addressbook_set_rubrica_and_flags(rubrica_get_rubrica(), 
					    FALSE, FALSE, FALSE, page);
  
  /* visualizzo il numero di indirizzi importati */
  rubrica_view_set_indirizzi(n);

  /* la rubrica  modificata */
  rubrica_set_modified(TRUE);

  rubrica_set_empty(FALSE);  
}


void rubrica_import_open_file(gchar *file)
{
  FILE *fp;

  if ((fp=fopen(file, "r")) == NULL)
    {
      g_print("\nerrore, impossibile leggere il file");
      rubrica_error_import_no_file();

      return;
    }

  while (!feof(fp))
    {
      gchar buffer[124];

      while (fgets(buffer, 124, fp))
	{	  
	  rubrica_import_gcard_decode_buffer(buffer);
	}
    }
}

void rubrica_export_data(void)
{
}


gint rubrica_import_open_old_rubrica_file(gchar *fname)
{
  RUBRICA_ERROR err;
  gboolean loaded;
  
  loaded = rubrica_file_load_xmlfile(fname, &err);

  if (!loaded)
    {
      rubrica_error_display_error(NULL, fname, err);      

      return 0;
    }
  
  return (gint) rubrica_view_get_indirizzi();    
}



