/*
*  Rubrica
*  file: rubrica.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <gconf/gconf-client.h>
#include <time.h>

#include "rubrica.h"
#include "view.h"
#include "modulo.h"
#include "prefer.h"
#include "main.h"
#include "file.h"
#include "addressbook.h"
#include "dictionary.h"


static gint range = 0;
static gint current_page;
static gboolean row_selected = FALSE;

Rubrica *current_rubrica;
RubricaItem *rtemp = NULL;


Rubrica *rubrica_rubrica_new(void)
{
  Rubrica *rubrica;

  rubrica = (Rubrica *) g_malloc(sizeof(Rubrica));

  if (!rubrica)
    g_error(_("Out of memory! I can't allocate enough memory for rubrica"));
  
  rubrica->rub      = NULL;
  rubrica->new      = TRUE;
  rubrica->empty    = TRUE;
  rubrica->modified = FALSE;
  rubrica->name     = NULL;
  rubrica->path     = NULL;
  rubrica->child    = NULL;
  rubrica->clist    = NULL;
  rubrica->label    = NULL;
  rubrica->items    = 0;
  rubrica->riga     = -1;
  rubrica->colonna  = 0;
  rubrica->pagina   = -1;  

  return (Rubrica*) rubrica;
}

void rubrica_init(GConfClient *client, gboolean first_run)
{
  gchar *dicthome = NULL;
  gchar *home, *homedir, *olddict;
  LANG language;

  g_print("\nRubrica: begin initialization");

  current_rubrica = rubrica_rubrica_new();
  current_page = 0;

  rubrica_addressbook_new();
  rubrica_init_gconf(client);
  rubrica_update_time();

  home     = g_get_home_dir();
  homedir  = g_strdup_printf("%s/.rubrica/dictionary", home);
  dicthome = rubrica_preferences_get_dictionary_home();

  if (!dicthome ||
      (g_strcasecmp(dicthome, homedir) != 0))
    {
      dicthome = g_strdup_printf("%s/.rubrica/dictionary", home);
      rubrica_preferences_set_dictionary_home(dicthome);
    }

  if (rubrica_preferences_get_use_default_dictionary())
    {
      if(rubrica_preferences_get_use_italian_dictionary())
	language = ITALIAN;
      else
	language = ENGLISH;
    }
  else
    language = USER;
  
  olddict = rubrica_preferences_get_string_key("/apps/rubrica/translation/user_dictionary_name");
  
  if (olddict && strcmp(olddict, "") != 0)
    {
      rubrica_preferences_set_user_dictionary(olddict);
      rubrica_preferences_set_string_key("/apps/rubrica/translation/user_dictionary_name", "");
    }
  
  if (first_run)
    rubrica_dictionary_init(ITALIAN);
  else
    rubrica_dictionary_init(language);  
  rubrica_addressbook_init();
 
  g_print("\nRubrica: initialization done...");
}

void 
rubrica_init_gconf(GConfClient *client)
{
  gconf_client_add_dir(client, "/apps/rubrica", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);  
  gconf_client_add_dir(client, "/apps/rubrica/file", 
		       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/general", 
		       GCONF_CLIENT_PRELOAD_NONE, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/translation", 
		       GCONF_CLIENT_PRELOAD_NONE, NULL);
  gconf_client_add_dir(client, "/apps/rubrica/dictionary", 
		       GCONF_CLIENT_PRELOAD_NONE, NULL);  
}


gboolean 
rubrica_start_with_user_addressbook()
{
  return rubrica_preferences_get_have_default_addressbook();
}


gchar*
rubrica_get_filename()
{
  gchar *file;
  
  file = g_strdup(rubrica_preferences_get_book());
  
  return (gchar *)(file ? file : NULL);
}


void 
rubrica_update_time()
{
  time_t last_run;

  last_run = time(NULL);

  rubrica_preferences_set_last_run(last_run);
}

void rubrica_set_current_page(gint n)
{
  current_page = n;
}

gint rubrica_get_current_page(void)
{
  return current_page;
}

void 
rubrica_copy_rubrica(Rubrica *r)
{
  current_rubrica->rub = r->rub;

  current_rubrica->new = r->new;
  current_rubrica->empty = r->empty;
  current_rubrica->modified = r->modified;
  
  if (current_rubrica->path)
    g_free(current_rubrica->path);   
  current_rubrica->path = g_strdup(r->path);
  
  if (current_rubrica->name)
    g_free(current_rubrica->name);
  current_rubrica->name = g_strdup(r->name);
  
  current_rubrica->child = r->child;
  current_rubrica->clist = r->clist;
  current_rubrica->label = r->label;
  
  current_rubrica->items = g_list_length(r->rub);
  current_rubrica->riga = r->riga;
  current_rubrica->colonna = r->colonna;

  current_rubrica->pagina = r->pagina;
}


void 
rubrica_save_rubrica(void)
{
  rubrica_addressbook_set_info(current_rubrica->rub, current_rubrica->new,
			       current_rubrica->modified, current_rubrica->empty,
			       current_rubrica->path, current_rubrica->name, 
			       current_rubrica->child, current_rubrica->clist, 
			       current_rubrica->label, current_rubrica->riga,
			       current_rubrica->colonna, TRUE,  TRUE, TRUE,
			       TRUE, TRUE, current_rubrica->pagina);
}


/* Metodi operanti sugli elementi della struttuta Rubrica */

/*   ritorna la testa della lista degli indirizzi
*/
GList *
rubrica_get_rubrica(void)
{
  return (GList*) current_rubrica->rub;
}


/*  ritorna:
    true se la lista  vuota
    false altrimenti
*/         
inline gboolean 
rubrica_is_empty(void)
{
  return (gboolean) current_rubrica->empty;
}

/*  ritorna:
    true se la lista  nuova
    false altrimenti
*/
inline gboolean 
rubrica_is_new(void)
{
  return (gboolean) current_rubrica->new;
}


/*  ritorna:
    true se la lista  stata modificata
    false altrimanti
*/
inline gboolean 
rubrica_is_modified(void)
{
  return (gboolean) current_rubrica->modified;
}

/*  ritorna:  la path della rubrica
*/
inline gchar*
rubrica_get_path(void)
{
  return (gchar *) current_rubrica->path;
} 


/*  ritorna il nome della rubrica
*/
inline gchar*
rubrica_get_name(void)
{
  return (gchar *) current_rubrica->name;
} 


/*  ritorna:
*/
inline GtkWidget*
rubrica_get_child(void)
{
  return (GtkWidget *) current_rubrica->child; 
}

/*  ritorna la clist in cui  mostrata la rubrica
*/
inline GtkWidget*
rubrica_get_clist(void)
{
  return (GtkWidget *) current_rubrica->clist; 
}


/*  ritorna il ptr all'etichetta che mosta il nome della rubrica
*/
inline GtkWidget*
rubrica_get_label(void)
{
  return (GtkWidget *) current_rubrica->label; 
}

/*  ritorna il numero di elementi nella rubrica
*/
inline gint 
rubrica_get_items(void)
{
  return (gint) current_rubrica->items;
}


/*  ritorna il numero (riga, posizione nella clist) di scheda che  mostrato
*/  
inline gint 
rubrica_get_row(void)
{
  return (gint) current_rubrica->riga;
}


/*  ritorna la colonna della scheda in cui si  cliccato
*/  
inline gint 
rubrica_get_column(void)
{
  return (gint) current_rubrica->colonna;
}

/*  ritorna il numero della pagina di notebook in cui 
    si trova la clist che mostra la rubrica
*/
inline gint 
rubrica_get_page(void)
{
  return (gint) rubrica_addressbook_get_current_page();
}


/**
 **
 **      Set
 **
 **/

/*   setta ptr come nuova testa della lista degli indirizzi
*/
void 
rubrica_set_rubrica(gpointer ptr)
{
  GList *lst = (GList *) ptr;
  gint page;

  page = rubrica_addressbook_get_current_page();

  current_rubrica->rub = lst;
  rubrica_addressbook_set_rubrica(lst, page);
}


/*  setta:
    true se la lista  nuova
    false altrimenti  
 */
inline void 
rubrica_set_new(gboolean state)
{
  gint page;

  page = rubrica_addressbook_get_current_page();

  current_rubrica->new = (gboolean) state;
  rubrica_addressbook_set_flag(state, TRUE, FALSE, FALSE, page);
}


/*  setta a:
    true se la lista  vuota
    false altrimenti  
 */
inline void 
rubrica_set_empty(gboolean stato)
{
  gint page;

  page = rubrica_addressbook_get_current_page();
  current_rubrica->empty = stato;
  rubrica_addressbook_set_flag(stato, FALSE, TRUE, FALSE, page);
}


/*  setta a:
    true se la lista  modificata
    false altrimenti  
 */
void 
rubrica_set_modified(gboolean modificata)
{
  GtkWidget *win;
  gchar *title, *name;
  gint page;
  
  win = rubrica_get_app();
  page = rubrica_addressbook_get_current_page();
  rubrica_addressbook_get_path_and_name(NULL, &name, page);
  
  if (modificata) 
    {
      title = g_strdup_printf(_("Rubrica -- %s changed --"), name);
      gtk_window_set_title(GTK_WINDOW(win), title);
      g_free(title);
    }
  else      
    {
      gtk_window_set_title(GTK_WINDOW(win), "Rubrica");
    }
  
  current_rubrica->modified = modificata;
  rubrica_addressbook_set_flag(modificata, FALSE, FALSE, TRUE, page);
}


/*  imposta la path a str
*/
void 
rubrica_set_path(gchar *str)
{
  gint page;

  page = rubrica_addressbook_get_current_page();

  if (current_rubrica->path)
    g_free(current_rubrica->path);

  current_rubrica->path = g_strdup(str);
  rubrica_addressbook_set_path(str, page);
}


/*  imposta a str il nuovo nome della rubrica
*/
void 
rubrica_set_name(gchar *str)
{
  gint page;

  page = rubrica_addressbook_get_current_page();
  
  if (current_rubrica->name)
    g_free(current_rubrica->name);

  current_rubrica->name = g_strdup(str);
  rubrica_addressbook_set_name(str, page);
}


/*  modifica il nome della rubrica visualizzato da label
*/
void 
rubrica_set_label_text(gchar *str)
{
  gint page;

  page = rubrica_addressbook_get_current_page();
  
  gtk_label_set_text(GTK_LABEL(current_rubrica->label), str); 
  rubrica_addressbook_set_label_text(str, page);
}



/*  setta il numero di elementi a n
 */
void 
rubrica_set_items(gint n)
{
  current_rubrica->items = n;
}

/*  setta il numero di riga selezionato a n
    
     il numero di riga della clist che
     attualmente selezionata
 */
void 
rubrica_set_row(gint n)
{
  current_rubrica->riga = n;
  rubrica_addressbook_set_row(n, current_page);
}

/*  imposta il numero di colonna a n
    
     il numero di colonna della clist
    che  stato selezionato
*/
void 
rubrica_set_column(gint n)
{
  gint page;

  page = rubrica_addressbook_get_current_page();

  current_rubrica->colonna = n;
  rubrica_addressbook_set_column(n, page);
}


/*
   ************************************************************************
   ************************************************************************
*/

void
rubrica_inserisci_scheda(RubricaItem *r)
{
  gchar *buffer[5];
  GtkWidget *pbar;
  gint pos;

  g_return_if_fail(r != NULL);
  
  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(r->scheda);
  buffer[2] = g_strdup(r->personal.first_name);
  buffer[3] = g_strdup(r->personal.last_name);
  buffer[4] = NULL;

  pbar = rubrica_view_get_progressbar();

  gtk_progress_set_value(GTK_PROGRESS(pbar),(float) current_rubrica->items);

  pos = gtk_clist_append(GTK_CLIST(current_rubrica->clist), buffer);
  gtk_clist_set_row_data(GTK_CLIST(current_rubrica->clist), pos, r);
}


void 
rubrica_elimina_scheda(gint n)
{
  GtkWidget *pbar;
  gint indirizzi;

  pbar = rubrica_view_get_progressbar();
  indirizzi = rubrica_view_get_indirizzi();

  /* il numero degli elementi  stato aggiornato nella 
     chiamata a rubrica_elimina_elemento
  */
  gtk_clist_remove(GTK_CLIST(current_rubrica->clist), n);  
  gtk_progress_set_value(GTK_PROGRESS(pbar), (float) indirizzi);  
}


void
rubrica_modifica_scheda(RubricaItem *r)
{
  gchar *buffer[5];
  gint n, pos;

  g_return_if_fail(r != NULL);

  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(r->scheda);
  buffer[2] = g_strdup(r->personal.first_name);
  buffer[3] = g_strdup(r->personal.last_name);
  buffer[4] = NULL;
  
  n = rubrica_get_row();

  gtk_clist_remove(GTK_CLIST(current_rubrica->clist), n);
  pos = gtk_clist_insert(GTK_CLIST(current_rubrica->clist), n, buffer);
  gtk_clist_set_row_data(GTK_CLIST(current_rubrica->clist), pos, r);  
}



/*   insert an item in addressbook
*/
void 
rubrica_inserisci_elemento(RubricaItem *r)
{
  current_rubrica->rub = g_list_append(current_rubrica->rub, r);
  current_rubrica->items++;

  if (rubrica_is_new())
    {
      gint page;

      page = rubrica_addressbook_get_current_page();

      rubrica_addressbook_set_rubrica(current_rubrica->rub, page);
    }    

  rubrica_set_new(FALSE);
}


/*  return the nth item in addressbook 
*/
RubricaItem*
rubrica_prendi_elemento(gint n)
{
  gint len;

  len = current_rubrica->items;

  if (n > len)
    return NULL;
  /* else  */
 return (RubricaItem *) g_list_nth_data(current_rubrica->rub, n);
}


/*    modify a rubrica's item
*/
gint 
rubrica_modifica_elemento(RubricaItem *r)
{
  gint n;
  GList *e = NULL;  
  gpointer new, old;

  g_return_val_if_fail(r != NULL, 0);

  new = r;
  n = current_rubrica->riga;

  e = g_list_nth(current_rubrica->rub, n);
  old = g_list_nth_data(current_rubrica->rub,n);
  
  e->data = new;
  g_free(old);
  
  return 1;
}


/*      free memory used by rubrica's item
*/
void 
rubrica_free_data(gpointer data, gpointer user_data)
{  
  g_return_if_fail(data != NULL);
  g_return_if_fail(user_data != NULL);
  
  if (user_data && user_data != "")
    g_free(user_data);
}


void 
rubrica_free(gchar *str)
{
  g_return_if_fail(str != NULL);

  if (strcmp(str, "") == 0)
    return;
  
  g_free(str);
}


/*      get the item that will be deleted
 */
gint 
rubrica_elimina_elemento(GList *rubrica, gint n)
{
  /*
    NOTA:
    La memoria occupata dalle liste si libera con 
    g_list_free_1 e non con g_free() pena SEGMENTATION FAULT
  */
  GList *nodo;
  GList *wl, *ml, *tl;
  RubricaItem *r;

  g_return_val_if_fail(rubrica != NULL, 0);

  /* prendo l'elemento alla posizione n */
  nodo = g_list_nth(rubrica, n);
  
  /* rimuovo l'elemento nodo dalla lista la memoria occupata da 
     nodo ancora non  liberata. Ora nodo  la testa di una 
     nuova lista, posso aggiungere altri elementi e costruire 
     una nuova lista, o cancellarlo elimindolo dalla lista
  */
  rubrica = g_list_remove_link(rubrica, nodo);

  /* estraggo i dati contenuti nel nodo e libero 
     la memoria che occupano 
  */
  r = (RubricaItem*) g_list_nth_data(nodo, 0);
  if (!r) 
    {
      g_warning("\ncan't extract selected node for deletion");

      return;
    }
  /*
    r  un ptr a RubricaItem, 
    che contiene diversi *gchar e 3 liste.   
  */
  wl = r->net.web_list;
  ml = r->net.mail_list;
  tl = r->telephone.tel_list;

  /* ridondanza di controlli??
     preferisco effettuare pi controlli ed evitare
     segmentation fault. Se qualcuno vuole reimplemantare
     in modo migliore questa parte di codice,  il
     benvenuto!! 
     
     If someone thinks that there are much controls, 
     well i think so. But i prefer this to a segmentation fault.
     Enyone can modify this code, to make it better. Thanks :-)	 

   */
  if (wl) {
    g_list_foreach(wl, rubrica_free_data, wl->data);
    g_list_free_1(r->net.web_list); 
  }
  
  if (ml) {
    g_list_foreach(ml, rubrica_free_data, ml->data);
    g_list_free_1(r->net.mail_list);
  }
  
  if (tl) {
    g_list_foreach(tl, rubrica_free_data, tl->data);
    g_list_free_1(r->telephone.tel_list);
  }

  rubrica_free(r->scheda);
  rubrica_free(r->personal.first_name);
  rubrica_free(r->personal.middle_name);
  rubrica_free(r->personal.last_name);
  rubrica_free(r->personal.profession);
  rubrica_free(r->personal.name_prefix);
  rubrica_free(r->personal.title);

  rubrica_free(r->address.street);
  rubrica_free(r->address.number);
  rubrica_free(r->address.zip_code);
  rubrica_free(r->address.city);
  rubrica_free(r->address.province);
  rubrica_free(r->address.state);
  rubrica_free(r->address.country);

  rubrica_free(r->company.name);
  rubrica_free(r->company.street);
  rubrica_free(r->company.number);
  rubrica_free(r->company.zip_code);
  rubrica_free(r->company.city);
  rubrica_free(r->company.province);
  rubrica_free(r->company.country);
  rubrica_free(r->company.web);
  rubrica_free(r->company.email);
  rubrica_free(r->company.operator);
  rubrica_free(r->company.fax);
  rubrica_free(r->company.green);
  rubrica_free(r->company.customer_care);
  rubrica_free(r->company.notes);

  rubrica_free(r->work.assigment);
  rubrica_free(r->work.organization);
  rubrica_free(r->work.department);
  rubrica_free(r->work.sub_dep);
  rubrica_free(r->work.secretary_name);
  rubrica_free(r->work.secretary_phone);

  rubrica_free(r->notes.spouse_name);
  rubrica_free(r->notes.child);
  rubrica_free(r->notes.hobby);
  rubrica_free(r->notes.notes);
  rubrica_free(r->notes.public_key);

  g_free(r);

  /* libero la menoria occupata da nodo */
  g_list_free_1(nodo);

  /* decremento il numero totale di indirizzi */
  rubrica_view_dec_indirizzi();
  return 1;
}


void 
rubrica_svuota_memoria(GList *rubrica)
{
  gint n;
  
  n = g_list_length(rubrica);
  
  while (n)
    {
      rubrica_elimina_elemento(rubrica, n-1);
      n--;
    }      
}


/* gestione visualizzazione dati schede
   
   TO DO:

   da spostare in un file dedicato alla visualizzazione dei dati 
   contenuti in rubrica
*/
void 
rubrica_mostra_scheda(RubricaItem *i)
{
  GtkCList *clist;
  GList *alias = NULL;
  gint n = 15;
  gint elementi = 0;
  gboolean modificato;              

  /* dati[..][0]  riservato per probabili future immagini
     dati[..][0] is reserved. Probably future pixmap
   */
  gchar *dati[54][4] = {
    {NULL, _("Card: "),                NULL, NULL},  /*  0 =>  1  */

    {NULL, _("First Name: "),          NULL, NULL},  /* 1 */       
    {NULL, _("Middle Name: "),         NULL, NULL},        
    {NULL, _("Last Name: "),           NULL, NULL},
    {NULL, _("Profession: "),          NULL, NULL},
    {NULL, _("Name Prefix: "),         NULL, NULL},
    {NULL, _("Title: "),               NULL, NULL},
    {NULL, _("Birthday: "),            NULL, NULL},  /* 7 */

    {NULL, _("Street: "),              NULL, NULL},  /* 8 */
    {NULL, _("Number: "),              NULL, NULL},
    {NULL, _("Zip code: "),            NULL, NULL},
    {NULL, _("City: "),                NULL, NULL},
    {NULL, _("Province: "),            NULL, NULL},
    {NULL, _("State: "),               NULL, NULL},
    {NULL, _("Country: "),             NULL, NULL},  /* 14 */
    
    {NULL, _("Web Pages: "),           NULL, NULL},  /* 15 */
    {NULL, _("Email Address: "),       NULL, NULL},
    {NULL, _("Home Telephone: "),      NULL, NULL},
    {NULL, _("Work Telephone: "),      NULL, NULL},
    {NULL, _("Fax Telephone: "),       NULL, NULL},
    {NULL, _("Cellphone: "),           NULL, NULL},
    {NULL, _("Other Telephone: "),     NULL, NULL},  /* 21 */

    {NULL, _("Assigment: "),           NULL, NULL},  /* 22 */
    {NULL, _("Organization: "),        NULL, NULL},
    {NULL, _("Department: "),          NULL, NULL},
    {NULL, _("Sub Department: "),      NULL, NULL},
    {NULL, _("Secretary Name: "),      NULL, NULL},
    {NULL, _("Secretary Telephone: "), NULL, NULL},  /* 27 */

    {NULL, _("Company Name: "),        NULL, NULL},  /* 28 */
    {NULL, _("Street: "),              NULL, NULL},
    {NULL, _("Street Number: "),       NULL, NULL},
    {NULL, _("Zip code: "),            NULL, NULL},
    {NULL, _("City: "),                NULL, NULL},
    {NULL, _("Provice: "),             NULL, NULL},
    {NULL, _("Country: "),             NULL, NULL},
    {NULL, _("Web: "),                 NULL, NULL},
    {NULL, _("Email: "),               NULL, NULL},
    {NULL, _("Operator: "),            NULL, NULL},
    {NULL, _("Fax: "),                 NULL, NULL},
    {NULL, _("Green: "),               NULL, NULL},
    {NULL, _("Customer Care: "),       NULL, NULL}, 
    {NULL, _("Notes: "),               NULL, NULL},  /* 41 */

    {NULL, _("Married: "),             NULL, NULL},  /* 42 */
    {NULL, _("Spouse Name: "),         NULL, NULL},   
    {NULL, _("Children: "),            NULL, NULL},   
    {NULL, _("Spuse Birthday: "),      NULL, NULL},   
    {NULL, _("Anniversary: "),         NULL, NULL},   
    {NULL, _("Children Notes: "),      NULL, NULL},   
    {NULL, _("Hobbies: "),             NULL, NULL},   
    {NULL, _("Public Key: "),          NULL, NULL},   
    {NULL, _("Notes: "),               NULL, NULL},  /* 50 */  

    {NULL, _("Card Created: "),        NULL, NULL},  /* 51 */
    {NULL, _("Last Change: "),         NULL, NULL}, 
    {NULL, NULL,                       NULL, NULL}   /* 53  => 54 */
  };
  
  g_return_if_fail(i != NULL);

  clist = rubrica_view_get_dati();
  gtk_clist_freeze(GTK_CLIST(clist));
  gtk_clist_clear(GTK_CLIST(clist));

  dati[0][2] = (i->scheda ? g_strdup(i->scheda): "");

  dati[1][2] = (i->personal.first_name  ? g_strdup(i->personal.first_name) : "");
  dati[2][2] = (i->personal.middle_name ? g_strdup(i->personal.middle_name) : "");
  dati[3][2] = (i->personal.last_name   ? g_strdup(i->personal.last_name) : "");
  dati[4][2] = (i->personal.profession  ? g_strdup(i->personal.profession) : "");
  dati[5][2] = (i->personal.name_prefix ? g_strdup(i->personal.name_prefix) : "");
  dati[6][2] = (i->personal.title       ? g_strdup(i->personal.title) : "");
  if (i->personal.birthday_setted)
    dati[7][2] = rubrica_convert_time(i->personal.birthday, FALSE);
  else
    dati[7][2] = _("unknown");
  dati[8][2]  = (i->address.street   ? g_strdup(i->address.street) : "");
  dati[9][2]  = (i->address.number   ? g_strdup(i->address.number) : "");
  dati[10][2] = (i->address.zip_code ? g_strdup(i->address.zip_code) : "");
  dati[11][2] = (i->address.city     ? g_strdup(i->address.city) : "");
  dati[12][2] = (i->address.province ? g_strdup(i->address.province) : "");
  dati[13][2] = (i->address.state    ? g_strdup(i->address.state) : "");
  dati[14][2] = (i->address.country  ? g_strdup(i->address.country) : "");
  
  rubrica_visualizza_dati(clist, dati);
  
  /*   web   */
  alias = g_list_first(i->net.web_list);
  elementi = i->net.nweb;
  modificato = FALSE;
  while (elementi > 0)
    {
      gint pos;

      if (alias && alias->data)
	dati[15][2] = g_strdup(alias->data);  
	  
      pos = gtk_clist_insert(GTK_CLIST(clist), n, dati[15]);
      gtk_clist_set_row_data(GTK_CLIST(clist), pos, dati[15][2]);	

      if (!modificato)
	{
	  dati[15][1] = g_strdup("");
	  modificato = TRUE;
	}
      n++;
      elementi--;
      alias = g_list_next(alias);
    }

  /*   email   */
  alias = g_list_first(i->net.mail_list);
  modificato = FALSE;
  elementi = i->net.nmail;
  while (elementi > 0)
    {
      gint pos;

      if (alias && alias->data)
	dati[16][2] = g_strdup(alias->data);  

      pos = gtk_clist_insert(GTK_CLIST(clist),n, dati[16]);
      gtk_clist_set_row_data(GTK_CLIST(clist), pos, dati[16][2]);

      if (!modificato)
	{
	  dati[16][1] = g_strdup("");
	  modificato = TRUE;
	}
      n++;
      elementi--;
      alias = g_list_next(alias);
    }

  /*   telefono   */
  alias = g_list_first(i->telephone.tel_list);
  elementi = i->telephone.ntel;
  while (elementi > 0)
    {
 
      TelNum *tel = NULL;
      TELEPHONE_TYPE tipo = TELEPHONE_UNKNOWN;

      if (alias && alias->data)     
	{
	  tel = (TelNum*) alias->data; 
	  if (tel && tel->tipo)
	    tipo = modulo_rubrica_tipo_telefono_int(tel->tipo);
	}

      if (!tel && (elementi > 0))
	elementi--;

      if (tel && tel->numero_telefono && (elementi > 0))
	{
	  switch (tipo)
	    {
	    case TELEPHONE_HOME:	  
	      dati[17][2] = g_strdup(tel->numero_telefono);
	      gtk_clist_insert(GTK_CLIST(clist), n, dati[17]);
	      n++;
	      elementi--;
	      alias = g_list_next(alias);
	      break;
	      
	    case TELEPHONE_WORK:
	      dati[18][2] =  g_strdup(tel->numero_telefono);
	      gtk_clist_insert(GTK_CLIST(clist), n, dati[18]);
	      n++;
	      elementi--;
	      alias = g_list_next(alias);
	      break;
	  
	    case TELEPHONE_FAX:
	      dati[19][2] = g_strdup(tel->numero_telefono);
	      gtk_clist_insert(GTK_CLIST(clist), n, dati[19]);
	      n++;
	      elementi--;
	      alias = g_list_next(alias);
	      break;
	      
	    case TELEPHONE_CELLPHONE:
	      dati[20][2] = g_strdup(tel->numero_telefono);
	      gtk_clist_insert(GTK_CLIST(clist), n, dati[20]);
	      n++;
	      elementi--;
	      alias = g_list_next(alias);
	      
	    case TELEPHONE_OTHER:
	      dati[21][2] = g_strdup(tel->numero_telefono);
	      gtk_clist_insert(GTK_CLIST(clist), n, dati[21]);
	      n++;
	      elementi--;
	      alias = g_list_next(alias);
	      break;
	      
	    case TELEPHONE_UNKNOWN:
	    default:
	      dati[21][2] = g_strdup(tel->numero_telefono);
	      gtk_clist_insert(GTK_CLIST(clist), n, dati[21]);
	      n++;
	      elementi--;
	      alias = g_list_next(alias);
	      break;
	    }
	}
    }  

  dati[22][2]  = (i->work.assigment       ? g_strdup(i->work.assigment): "");
  dati[23][2]  = (i->work.organization    ? g_strdup(i->work.organization): "");
  dati[24][2]  = (i->work.department      ? g_strdup(i->work.department): "");
  dati[25][2]  = (i->work.sub_dep         ? g_strdup(i->work.sub_dep): "");
  dati[26][2]  = (i->work.secretary_name  ? g_strdup(i->work.secretary_name): "");
  dati[27][2]  = (i->work.secretary_phone ? g_strdup(i->work.secretary_phone): "");

  dati[28][2]  = (i->company.name     ? g_strdup(i->company.name): "");
  dati[29][2]  = (i->company.street   ? g_strdup(i->company.street): "");
  dati[30][2]  = (i->company.number   ? g_strdup(i->company.number): "");
  dati[31][2]  = (i->company.zip_code ? g_strdup(i->company.zip_code): "");
  dati[32][2]  = (i->company.city     ? g_strdup(i->company.city): "");
  dati[33][2]  = (i->company.province ? g_strdup(i->company.province): "");
  dati[34][2]  = (i->company.country  ? g_strdup(i->company.country): "");
  dati[35][2]  = (i->company.web      ? g_strdup(i->company.web): "");
  dati[36][2]  = (i->company.email    ? g_strdup(i->company.email): "");
  dati[37][2]  = (i->company.operator ? g_strdup(i->company.operator): "");
  dati[38][2]  = (i->company.fax      ? g_strdup(i->company.fax): "");
  dati[39][2]  = (i->company.green    ? g_strdup(i->company.green): "");
  dati[40][2]  = (i->company.customer_care   ? g_strdup(i->company.customer_care): "");
  dati[41][2]  = (i->company.notes    ? g_strdup(i->company.notes): "");

  dati[42][2]  = (i->notes.married     ? _("is married") : _("is not married"));
  dati[43][2]  = (i->notes.spouse_name ? g_strdup(i->notes.spouse_name): "");
  dati[44][2]  = (i->notes.children    ? _("has children") : _("hasn't children"));
  if (i->notes.married)
    {
      dati[45][2]  = rubrica_convert_time(i->notes.birthday, FALSE);  
      dati[46][2]  = rubrica_convert_time(i->notes.anniversary, FALSE);
    }
  else
    {
      dati[45][2] = "";
      dati[46][2] = "";
    }
  dati[47][2]  = (i->notes.child       ? g_strdup(i->notes.child): "");
  dati[48][2]  = (i->notes.hobby       ? g_strdup(i->notes.hobby): "");
  dati[49][2]  = (i->notes.public_key  ? g_strdup(i->notes.public_key): "");
  dati[50][2]  = (i->notes.notes       ? g_strdup(i->notes.notes): "");

  dati[51][2] = rubrica_convert_time(i->created, TRUE);
  dati[52][2] = rubrica_convert_time(i->last_change, TRUE);

  rubrica_visualizza_dati_rimanenti(clist, dati, n);

  gtk_clist_thaw(GTK_CLIST(clist));
}


/*   mostra i dati contenuti nella tabella "dati"
     dalla riga 0 alla riga 10
*/
void 
rubrica_visualizza_dati(GtkCList *clist, gchar *item[53][4])
{
  gint i;

  for (i=0; i<15; i++)
    gtk_clist_insert(GTK_CLIST(clist),i, item[i]);
}


void 
rubrica_visualizza_dati_rimanenti(GtkCList *clist, gchar *item[53][4], gint pos)
{
  gint i;
  
  for (i=22; i < 53; i++)
    gtk_clist_insert(GTK_CLIST(clist), pos++, item[i]);
}

/*  -----------------------------------------------------------------  */

/*    routine per la nagivazione delle schede inserite
*/
void 
rubrica_mostra_prima_scheda()
{
  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(current_rubrica->clist), 0, 0);  
  rubrica_set_row_selected(FALSE);
}

void 
rubrica_mostra_ultima_scheda()
{
  guint n;

  n = g_list_length(current_rubrica->rub)-1;

  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(current_rubrica->clist), n, 0);  
  rubrica_set_row_selected(FALSE);
}

void 
rubrica_scheda_ennesima(gint n)
{
  rubrica_set_row_selected(TRUE);
  gtk_clist_select_row(GTK_CLIST(current_rubrica->clist), n, 0);  
  rubrica_set_row_selected(FALSE);
}


/*    cut, 
      copy and
      paste      
*/
void 
rubrica_item_copy(void)
{
  GtkWidget *errdlg;
  GtkWidget *clist;
  RubricaItem *r;
  GList *rubrica;
  gchar *err;
  gint range;
  gint riga;
 
  if (rubrica_is_empty())
    {
      rubrica_error_rubrica_empty();
      
      return;
    }
  
  range = rubrica_get_range();

  if (!range)
    {
      rubrica_error_selezionare_elemento();
      
      return;
    }
  
  clist = rubrica_get_clist();
  riga  = rubrica_get_row();
  rubrica = rubrica_get_rubrica();
  
  rtemp = g_malloc(sizeof(RubricaItem));
  r = (RubricaItem*) g_list_nth_data(rubrica, riga);
//  r = (RubricaItem*) gtk_clist_get_row_data(GTK_CLIST(clist), riga);

  if (!r)        
    {
      err = g_strdup(_("I can't find data in this card"));
      errdlg = gnome_error_dialog(err);
      gtk_widget_show(errdlg);

      return;
    }
  
  rtemp->net.web_list = rtemp->net.mail_list = rtemp->telephone.tel_list = NULL;

  rtemp->editabile = r->editabile;
  rtemp->cancellabile = r->cancellabile;
  
  rtemp->net.nweb  = r->net.nweb;
  rtemp->net.nmail = r->net.nmail;
  rtemp->telephone.ntel  = r->telephone.ntel;
  
  rtemp->scheda = (r->scheda ? g_strdup(r->scheda) : "");
  
  rtemp->personal.first_name  = (r->personal.first_name  ? 
				 g_strdup(r->personal.first_name) : "");
  rtemp->personal.middle_name = (r->personal.middle_name 
				 ? g_strdup(r->personal.middle_name): "");
  rtemp->personal.last_name   = (r->personal.last_name ? 
				 g_strdup(r->personal.last_name): "");
  rtemp->personal.profession  = (r->personal.profession ? 
				 g_strdup(r->personal.profession): "");
  rtemp->personal.name_prefix = (r->personal.name_prefix ? 
				 g_strdup(r->personal.name_prefix): "");
  rtemp->personal.title       = (r->personal.title ? 
				 g_strdup(r->personal.title): "");
  rtemp->personal.birthday    = r->personal.birthday;
  
  rtemp->address.street   = (r->address.street   ? g_strdup(r->address.street): "");
  rtemp->address.number   = (r->address.number   ? g_strdup(r->address.number): "");
  rtemp->address.zip_code = (r->address.zip_code ? g_strdup(r->address.zip_code): "");
  rtemp->address.city     = (r->address.city     ? g_strdup(r->address.city): "");
  rtemp->address.province = (r->address.province ? g_strdup(r->address.province): "");
  rtemp->address.state    = (r->address.state    ? g_strdup(r->address.state): "");
  rtemp->address.country  = (r->address.country  ? g_strdup(r->address.country): "");

  while (r->net.web_list)
    {
      rtemp->net.web_list = g_list_append(rtemp->net.web_list, r->net.web_list->data);
      r->net.web_list = g_list_next(r->net.web_list);
    }

  while (r->net.mail_list)
    {
      rtemp->net.mail_list = g_list_append(rtemp->net.mail_list,r->net.mail_list->data);
      r->net.mail_list = g_list_next(r->net.mail_list);
    }  

  while (r->telephone.tel_list)
    {
      rtemp->telephone.tel_list = 
	g_list_append(rtemp->telephone.tel_list, r->telephone.tel_list->data);
      r->telephone.tel_list = g_list_next(r->telephone.tel_list);
    }

  rtemp->work.assigment       = (r->work.assigment  ? g_strdup(r->work.assigment): "");
  rtemp->work.organization    = (r->work.organization ? 
				 g_strdup(r->work.organization): "");
  rtemp->work.department      = (r->work.department ? g_strdup(r->work.department): "");
  rtemp->work.sub_dep         = (r->work.sub_dep ? g_strdup(r->work.sub_dep): "");
  rtemp->work.secretary_name  = (r->work.secretary_name ? 
				 g_strdup(r->work.secretary_name): "");
  rtemp->work.secretary_phone = (r->work.secretary_phone ? 
				 g_strdup(r->work.secretary_phone): "");
  
  rtemp->company.name     = (r->company.name     ? g_strdup(r->company.name): "");
  rtemp->company.street   = (r->company.street   ? g_strdup(r->company.street): "");
  rtemp->company.number   = (r->company.number   ? g_strdup(r->company.number): "");
  rtemp->company.zip_code = (r->company.zip_code ? g_strdup(r->company.zip_code): "");
  rtemp->company.city     = (r->company.city     ? g_strdup(r->company.city): "");
  rtemp->company.province = (r->company.province ? g_strdup(r->company.province): "");
  rtemp->company.country  = (r->company.country  ? g_strdup(r->company.country): "");
  rtemp->company.web      = (r->company.web      ? g_strdup(r->company.web): "");
  rtemp->company.email    = (r->company.email    ? g_strdup(r->company.email): "");
  rtemp->company.operator = (r->company.operator ? g_strdup(r->company.operator): "");
  rtemp->company.fax      = (r->company.fax      ? g_strdup(r->company.fax): "");
  rtemp->company.green    = (r->company.green    ? g_strdup(r->company.green): "");
  rtemp->company.customer_care = (r->company.customer_care ? 
				  g_strdup(r->company.customer_care): "");
  rtemp-> company.notes = (r->company.notes ? g_strdup(r->company.notes): "");
  
  rtemp->notes.married  = r->notes.married;
  rtemp->notes.spouse_name= (r->notes.spouse_name ? g_strdup(r->notes.spouse_name): "");
  rtemp->notes.children = r->notes.children;
  rtemp->notes.birthday = r->notes.birthday;  
  rtemp->notes.anniversary = r->notes.anniversary;
  rtemp->notes.child  = (r->notes.child ? g_strdup(r->notes.child): "");
  rtemp->notes.hobby  = (r->notes.hobby ? g_strdup(r->notes.hobby): "");
  rtemp->notes.notes  = (r->notes.notes ? g_strdup(r->notes.notes): "");
  rtemp->notes.public_key = (r->notes.public_key ? g_strdup(r->notes.public_key): "");
  
  rtemp->created     = r->created;
  rtemp->last_change = r->last_change;
  
  /* aggiungere il codice per inserire in undo/redo la "copiatura dell'elemento */
}


void 
rubrica_item_cut(void)
{
  GList *nodo;
  GList *rubrica;
  gint riga;

  if (rubrica_is_empty())
    {
      rubrica_error_rubrica_empty();
      
      return;
    }

  rubrica = rubrica_get_rubrica();
  riga = rubrica_get_row();

  nodo = g_list_nth(rubrica, riga);
  rubrica = g_list_remove_link(rubrica, nodo);
  rtemp = (RubricaItem*) g_list_nth_data(nodo, 0);

  rubrica_elimina_scheda(riga);
  rubrica_set_modified(TRUE);
  rubrica_view_dec_indirizzi();
}


void 
rubrica_item_paste(void)
{
  GList *rubrica;
  GtkWidget *errdlg;
  GtkWidget *clist;
  gchar *buffer[5];
  gchar *err;
  gint row;
  gint page;
  
  rubrica = rubrica_get_rubrica();
  clist   = rubrica_get_clist();
  row     = rubrica_get_row();
  page    = rubrica_get_page();

  if (!rtemp)
    {
      err = g_strdup(_("I can't paste anything if you don't cut/copy before"));
      errdlg = gnome_error_dialog(err);
      gtk_widget_show(errdlg);
    }
  
  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(rtemp->scheda);
  buffer[2] = g_strdup(rtemp->personal.first_name);
  buffer[3] = g_strdup(rtemp->personal.last_name);
  buffer[4] = NULL;


  if (range)
    {
      gtk_clist_insert(GTK_CLIST(clist), row, buffer);
      rubrica = g_list_insert(rubrica, rtemp, row);
    }
  else
    {
      row = gtk_clist_append(GTK_CLIST(clist), buffer);
      rubrica_inserisci_elemento(rtemp); 
    }

  gtk_clist_set_row_data(GTK_CLIST(clist), row, rtemp);
 
  rubrica_view_inc_indirizzi();
  rubrica_set_modified(TRUE);
  rubrica_addressbook_set_flags(TRUE, FALSE, FALSE, page);
}

gchar*
rubrica_convert_time(time_t t, gboolean has_hour)
{
  struct tm *loctime;
  gchar *buffer;
  
  buffer = g_malloc(sizeof(gchar)*256);
  
  if (t == 0)
    {
      buffer = "";
      return buffer;
    }

  loctime = localtime(&t);
  if (has_hour)
    strftime(buffer, 256, "%m/%d/%Y %H:%M:%S %Z", loctime);
  else
    strftime(buffer, 256, "%m/%d/%Y", loctime);
 
  return buffer;     
}


void rubrica_set_range(gint n)
{
  range = n;
}

gint rubrica_get_range(void)
{
  return range;
}

gboolean rubrica_other_are_modified()
{
  return rubrica_addressbook_heve_modified();
}

void rubrica_set_row_selected(gboolean bool)
{
  row_selected = bool;
}

gboolean rubrica_is_row_selected(void)
{
  return row_selected;
}

